/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file channel.h
 * @brief Channel header file
 */

#ifndef CHANNEL_H
#define CHANNEL_H

#include "tools.h"
#include <vector>
#include <string>
#include <iostream>
using namespace std;

/** @brief Channel management class
 *
 * This class stores and manage all informations 
 * about a channel
 */

class Channel
{
   public:
		/// Constructor
      Channel(string);
		/// Destructor
      ~Channel();
		/// return the channel name
		string getName();
		/// Add a user to the chan
		bool addUser(string,string,string,string);
		/// Del a user from the chan
		bool delUserByNick(string);
		/// Del a user from the chan
		bool delUserByHost(string);
      /// Return infos about a user
      string * getInfosByNick(string);
		/// Return the nick associated to a given host
		string getNickByHost(string);
		/// Return the status associated to a given nick
		string getStatusByNick(string);
		/// Return the status associated to a given host
		string getStatusByHost(string);
		/// Return the host associated to a given nick
		string getHostByNick(string);
		/// Return the ident associated to a nick
		string getIdentByNick(string);
		/// Return the ident associated to a host
		string getIdentByHost(string);
		/// Change a nick
		bool setNickByNick(string,string);
		/// Change the nick associated to a host
		bool setNickByHost(string,string);
		/// Update the status associated to a nick
		bool updateStatusByNick(string,char,char);
		/// Check if a nick owns a given access
		bool checkNickAccess(string,char);
		/// Erase all users from the channel
		void truncateUsersList();
		/// Gives users vector
		vector<string*> getUsers();
		/// Get last update
		time_t getLastWhoUpdate();
		/// Notify a WHO update
		void notifyWho();
		/// get topic
		string getTopic();
		/// set topic
		void setTopic(string);
      /// get informations about the last user who left channel
      string* getLastPartInfos();
      /// Tell if a nick is on the channel
      bool isOnChannel(string);
   private:
		/// Contain all the chan users
		vector<string*> users; // nick host ident status
		/// Return a nick iterator in a vector
		vector<string*>::iterator getIterator(string,unsigned int);
		/// Channel name
		string name ;
		/// Keep last WHO update
		time_t lastWhoUpdate;
		/// stores channel's topic
		string topic;
      /// Stores informations about the last user who left channel
      string lastPart[4];
};

#endif
