/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file configurationfile.h
 * @brief ConfigurationFile header file
 */

#ifndef CONFIGURATIONFILE_H
#define CONFIGURATIONFILE_H

#include "tools.h"
#include <string>
#include <fstream>
#include <map>
using namespace std;

/** @brief Configuration file class
 *
 * Class that manage a configuration file.
 * It uses a MAP containing values associated to keys
 */

class ConfigurationFile
{
   public:
		/// Class constructor
      ConfigurationFile(string);
		/// Class destructor
      ~ConfigurationFile();
		/// Parse and load the configuration file
		bool load();
		/// Flush the settings in the file
		bool flush();
		/// Return the configuration (keys and values, in a MAP container)
		map<string,string> getConfig();
      /// Add a key to protected list
      void addProtectedKey(string);
		/// Return a value associated to a key
		string getValue(string,bool displayProtected=true);
		/// Set a value to a key
		void setValue(string,string);
		/// Delete a key
		bool delKey(string);
      /// Returns file path
      string getFilePath();
	private:
		/// Configuration container
		map<string,string> config;
		/// Configuration file  path (and name)
		string file;
      /// Stores protected keys
      vector<string> protectedKeys;
};

#endif
