/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file ircprotocol.h
 * @brief IRCProtocol header file
 */

#ifndef IRCPROTOCOL_H
#define IRCPROTOCOL_H
#include <string>
#include <vector>
using namespace std;

/** @brief Class that convert messages to IRC messages
 *
 * This class convert "humain" strings to IRC messages
 * This class allow the bot to connect to different servers type 
 * simply by changing its code methods
 * All methods a static, no object needed
 */


class IRCProtocol {
public:
	/// Constructor
   IRCProtocol();
	/// Destructor
   ~IRCProtocol();
	/// Construct a string to identify to an irc server
   static vector<string> identify(string,string,string,string); 	
	/// Construct a string to quit an irc server
   static string quitServer(string raison = "autokilled!") ;
	/// Construct a string to join a channel
   static string joinChannel(string) ;
	/// Construct a string to leave a channel
   static string leaveChannel(string,string raison = "\\o_"); 
	/// Construct a string to change nick
   static string changeNick(string); 
	/// Construct a string to ping the server
   static string ping(string);
	/// Construct a string to respond to a ping
   static string pong(string); 
	/// Construct a string to send a message
   static string sendMsg(string,string); 
	/// Construct a vector with strings to send several messages
   static vector<string> sendMsg(string,vector<string>); 
   /// Construct a string to send an action on a channel
   static string sendAction(string,string);
	/// Construct a string to change a channel topic
   static string changeTopic(string,string); 
   /// Construct strings to apply a mode on different people
   static vector<string> applyModes(string,vector<string>,char,char,unsigned int);
	/// Construct strings to op people on a channel
   static vector<string> op(vector<string>,string); 
	/// Construct a string to op a user on a channel
   static string op(string,string); 
	/// Construct strings to unop people on a channel
   static vector<string> unop(vector<string>,string); 
	/// Construct a string to unop a user on a channel
   static string unop(string,string); 
	/// Construct strings to voice people on a channel
   static vector<string> voice(vector<string>,string); 
	/// Construct a string to voice a user on a channel
   static string voice(string,string);
	/// Construct strings to unvoice people on a channel
   static vector<string> unvoice(vector<string>,string); 
	/// Construct a string to unvoice a user on a channel
   static string unvoice(string,string); 
   /// Construct a string to ban a host on a channel
   static string ban(string,string);
   /// Construct a string to unban a host on a channel
   static string unban(string,string);
	/// Construct a string to send a notice
   static string sendNotice(string,string); 
	/// Construct strings to send notices
   static vector<string> sendNotices(string,vector<string>); 
	/// Construct a string to kick someone from channel
   static string kick(string,string,string); 
	/// Construct a string for a WHO command
	static string who(string,string);
   /// Construct a string for a INVITE command
   static string invite(string,string);
};

#endif
