/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file logfile.h
 * @brief LogFile header file
 */

#ifndef LOGFILE_H
#define LOGFILE_H

#include <sys/stat.h>
#include <strings.h>
#include <string>
#include <fstream>
#include <iostream>
#include <map>
using namespace std;

/// Log levels
typedef enum {NOTUSED,NOTHING,ERROR,WARNING,INFO} log_level;

/** @brief Class that manage log system
 *
 * This class is used to log bot events.
 * Differents levels are available :
 * error, warning, info and nothing
 */

class LogFile {
   public :
      /// Constructor
      LogFile(string,bool,bool,string,string);
      /// Destructor
      ~LogFile();
      /// Open log file
      bool open();
      /// Close log file
      void close();
      /// Close and open log file
      void reopen();
      /// Log an event
      bool log(string line,log_level ll=NOTUSED);
      /// Set period format
      void setPeriodFormat(string);
      /// Get period format
      string getPeriodFormat();
      /// Set verbose state
      void setVerbose(bool);
      /// Get verbose state
      bool getVerbose();
      /// Set keepfiles state
      void setKeepFiles(bool);
      /// Get keepfiles state
      bool getKeepFiles();
      /// Get log level
      log_level getLogLevel();
      /// Set log level
      void setLogLevel(log_level);
      /// Set log level
      void setLogLevel(string);
   private :
      /// File stream
      ofstream* stream;
      /// log level
      log_level level;
      /// Base file name
      string baseFileName;
      /// Period used
      string period;
      /// Period format
      string periodFormat;
      /// Get system date according to periodFormat
      string systemPeriod();
      /// keepFiles state
      bool keepFiles;
      /// Verbose state
      bool verbose;
      /// Init log file
      void beginLog();
      /// Finish log file
      void endLog();
      /// Check if log file exists
      bool checkFile();
      /// Get level tag for a given level
      string getLevelTag(log_level);
      /// Get a log level according to a string
      log_level strToLogLevel(string);
};

#endif
