/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file main.cpp
 * @brief Main program.
 *
 * Construct a botkernel and launch it.
 */

#include "botkernel.h"
#include "cppthread.h"

vector<string> listConfFiles(string);
void launchThreads(vector<string>,vector<CPPThread*>*,vector<BotKernel*>*);
void * launchBot(void *);
void displayHelp(string,bool);

int main(int nbArgs, char *arrayArgs[])  { 
   srand((unsigned)time(NULL));
   char * pHOME = NULL;
   pid_t pid;
   string confDir = "" ;
	unsigned char optchar;
   int opt;
	bool background=false;
   vector<CPPThread*> threads;
   vector<BotKernel*> bots;
	opterr = 0 ;	
   pHOME = getenv("HOME");
   if ( pHOME != NULL ) {
      confDir = (string)pHOME+"/.trustyrc/";
   }
	while ( (opt = getopt(nbArgs,arrayArgs,"-bc:")) != EOF ) {
      optchar = opt;
		switch (optchar) {
			case 'c' : confDir = optarg;
			break;
			case 'b' : background = true;
			break;
			case 1 : cout << "Invalid argument : "<<  optarg  << endl;
                  displayHelp(arrayArgs[0],true);
			break;
			default: 
			case '?' : cout << "Invalid option : "<< (char) optopt << endl;
                    displayHelp(arrayArgs[0],true);
			break;
		}	
	}
	if (background) {
      pid =  fork();
      switch (pid) {
         case -1 : cout << "Fork ERROR !, exiting" << endl;
         exit(-1);
         break;
         case 0 : // son
            setsid();
            launchThreads(listConfFiles(confDir),&threads,&bots);
         break;
         default :  // father 
            exit(0);
      }
   }
   else {
      launchThreads(listConfFiles(confDir),&threads,&bots);
   }
   // Wait for threads and delete them
   for(unsigned int i = 0 ; i < threads.size() ; i ++) {
      threads[i]->join();
      delete(threads[i]);
      delete(bots[i]);
   }
   return 0;
}

vector<string> listConfFiles(string confDir) {
   DIR *dp;
   struct dirent *ep;
   dp = opendir (confDir.c_str());
   vector<string> files;
   string::size_type pos ;
   if (dp != NULL) {
      while ((ep = readdir(dp))) {
         pos = ((string)ep->d_name).find(".") ;
         if ( (pos != string::npos) && (((string)ep->d_name).substr(pos)==".conf") ) {
            files.push_back(confDir+"/"+ep->d_name);
         }
      }
      closedir (dp);
   }
   else {
      cerr << "Couldn't open configuration directory. Exiting ..." << endl;
      exit(0);
   }
   return files;
}

void launchThreads(vector<string> confFiles,vector<CPPThread*>* threads,vector<BotKernel*>* bots) {
   for(unsigned int i = 0 ; i < confFiles.size() ; i ++ ) {
      threads->push_back(new CPPThread());
      bots->push_back(new BotKernel(confFiles[i]));
      threads->at(i)->exec(launchBot,bots->at(i));
   }
}

void * launchBot(void * arg) {
   ((BotKernel*)arg)->run();
   return NULL;
}

void displayHelp(string firstArg,bool quit) {
   cout << "Usage : "<< firstArg << " [-c configuration_dir] [-b]" << endl;
   if ( quit ) {
      exit(0);
   }
}
