/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file message.h
 * @brief Message header file
 */

#ifndef MESSAGE_H
#define MESSAGE_H
#include "tools.h"
#include <string>
#include <iostream>
using namespace std;

/** @brief Class that manage messages from the irc server
 *
 * This class stores messages from the irc server and parse them
 * to make informations easier to obtain.
 * Please be carefull. All methods works with message comming FROM the server,
 * not for those that are sended TO the server
 */

class Message {
   public :
		/// Constructor
      Message(string);
      /// Constructor
      Message();
		/// Destructor
      ~Message();
		/// Return all parts of the message
		vector<string> getSplit();
      /// Set the message string
      void setMessage(string);
      /// Return parts number
      unsigned int nbParts();
		/// Return a part of the message
		string getPart(unsigned int);
		/// Return the message's sender's informations (nick,ident,host)
		string getSender();
		/// Message's sender's nick
		string getNickSender();
		/// Message's sender's host
		string getHostSender();
		/// Message's sender's ident
		string getIdentSender();
		/// True if the message is a private one
		bool isPrivate();
		/// True if the message is a public one (channel)
		bool isPublic();
		/// Return the message source (channel or botnick)
		string getSource();
		/// Return the raw msg (irc format)
		string getMessage();
      /// Get elapsed time between message creation and now
      time_t getElapsedTime();
   private :
		/// The raw message
      string message;
		/// Message splitted (by spaces)
      vector<string> split ;
		/// True oif the message is private
		bool pv;
      /// timestamp of the message
      time_t timestamp;
};
#endif
