/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file admin.cpp
 * @brief Admin implementation file
 */

#include "admin.h"

/**
 * Constructor
 */ 
Admin::Admin(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "bot admin management";
	this->version = VERSION;
	this->name = "admin";
	this->bindFunction("addsuperadmin",IN_COMMAND_HANDLER,"addsuperadmin",0,10);
	this->bindFunction("addtempsuperadmin",IN_COMMAND_HANDLER,"addtempsuperadmin",0,10);
	this->bindFunction("superadminlist",IN_COMMAND_HANDLER,"superadminlist",0,10);
	this->bindFunction("setsuperadminpass",IN_COMMAND_HANDLER,"setSuperAdminPass",0,10);
	this->bindFunction("35",IN_LOOP,"clearTemporaryAdmins",0,30); 
	this->bindFunction("delsuperadmin",IN_COMMAND_HANDLER,"delsuperadmin",0,10);
	this->bindFunction("reset",IN_COMMAND_HANDLER,"reset",0,10);
	this->bindFunction("disconnect",IN_COMMAND_HANDLER,"disconnect",0,10);
	this->bindFunction("getconfvalue",IN_COMMAND_HANDLER,"getconfvalue",0,10);
	this->bindFunction("setconfvalue",IN_COMMAND_HANDLER,"setconfvalue",0,10);
	this->bindFunction("deletekey",IN_COMMAND_HANDLER,"deletekey",0,10);
	this->bindFunction("loadconffile",IN_COMMAND_HANDLER,"loadconffile",0,10);
	this->bindFunction("flushconffile",IN_COMMAND_HANDLER,"flushconffile",0,10);
	this->bindFunction("addonlyon",IN_COMMAND_HANDLER,"addOnlyon",0,10);
	this->bindFunction("delonlyon",IN_COMMAND_HANDLER,"delOnlyon",0,10);
	this->bindFunction("enable",IN_COMMAND_HANDLER,"enable",0,10);
	this->bindFunction("disable",IN_COMMAND_HANDLER,"disable",0,10);
	this->bindFunction("commandsstatus",IN_COMMAND_HANDLER,"commandsStatus",0,10);
	this->bindFunction("",IN_BEFORE_TREATMENT,"allowedCommandCheck",0,10);
	this->bindFunction("chanlev",IN_COMMAND_HANDLER,"chanlev",0,10);
	this->bindFunction("join",IN_COMMAND_HANDLER,"joinChannel",0,10);
	this->bindFunction("part",IN_COMMAND_HANDLER,"leaveChannel",0,10);
	this->bindFunction("cycle",IN_COMMAND_HANDLER,"cycleChannel",0,10);
	this->bindFunction("setnick",IN_COMMAND_HANDLER,"setNick",0,10);
	this->bindFunction("tell",IN_COMMAND_HANDLER,"tell",0,10);
	this->bindFunction("notice",IN_COMMAND_HANDLER,"notice",0,10);
	this->bindFunction("raw",IN_COMMAND_HANDLER,"raw",0,10);
	this->bindFunction("whoami",IN_COMMAND_HANDLER,"whoami",0,10);
	this->bindFunction("reauth",IN_COMMAND_HANDLER,"reauth",0,10);
	this->bindFunction("INVITE",IN_TYPE_HANDLER,"onInvite",0,10);
	this->bindFunction("ERROR",IN_FIRST_WORD,"error",0,10);
	this->bindFunction("setloglevel",IN_COMMAND_HANDLER,"setloglevel",0,10);
	this->bindFunction("setlogkeepfiles",IN_COMMAND_HANDLER,"setlogkeepfiles",0,10);
	this->bindFunction("setlogperiod",IN_COMMAND_HANDLER,"setlogperiod",0,10);
	this->bindFunction("getnbcountdowns",IN_COMMAND_HANDLER,"getnbcountdowns",0,10);
	this->bindFunction("clearCountDowns",IN_COMMAND_HANDLER,"clearCountDowns",0,10);
   this->doc = new TiXmlDocument(b->getDatasDir()+"admin.xml");
   if ( this->doc->LoadFile() )
   {
      this->root = this->doc->FirstChild();
   }
   else
   {
      this->initFile();
   }
}

/**
 * Initilaize the XML file by creating root and first childs (file empty structure)
 */
void Admin::initFile()
{
   TiXmlElement born("trustyrc_access");
   this->doc->InsertEndChild(born);
   this->root = this->doc->FirstChild();
   TiXmlElement channelsNode( "channels" );
   this->root->InsertEndChild(channelsNode);
   TiXmlElement superAdminsNode("super_admins" );
   this->root->InsertEndChild(superAdminsNode);  
   TiXmlElement disableNode("disabled_commands" );
   this->root->InsertEndChild(disableNode); 
   TiXmlElement onlyonNode("onlyon_commands" );
   this->root->InsertEndChild(onlyonNode); 
   this->doc->SaveFile();
}

/**
 * Add a channel managed by the bot
 * @param channel Channel name that you want to add
 * @return true if the channel has been added, else false
 */
bool Admin::addChannel(string channel)
{
   if(this->channelExists(channel)) {
      return false;
   }
   else {
      TiXmlElement elem("channel");
      elem.SetAttribute("name",Tools::to_lower(channel));
      this->root->FirstChild("channels")->InsertEndChild(elem) ;
      this->doc->SaveFile();
      return true;
   }
}

/**
 * Remove a channel managed by the bot
 * @param channel Channel name that you want to delete
 * @return true if the channel has been deleted, else false
 */
bool Admin::delChannel(string channel)
{
   bool back=false;
   TiXmlElement * elem;
   elem = this->root->FirstChild("channels")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( Tools::to_lower(elem->Attribute("name")) == Tools::to_lower(channel) )
      {
         back = this->root->FirstChild("channels")->RemoveChild(elem) ;
         this->doc->SaveFile();
         return back;
      }
      elem = elem->NextSiblingElement();
   }
   return back;
}

/**
 * Tell if a channel is registred
 * @param channel Channel name that you want to test existance
 * @return true if the channel exists, else false
 */
bool Admin::channelExists(string channel)
{
   TiXmlElement * elem;
   elem = this->root->FirstChild("channels")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( Tools::to_lower(elem->Attribute("name")) == Tools::to_lower(channel) )
         return true;
      elem = elem->NextSiblingElement();
   }
   return false;
}

/**
 * Add a super admin for the bot
 * @param mask Super admin's mask
 * @return True if the admin has been added, else false
 */
bool Admin::addSuperAdmin(string mask)
{
   if(this->isSuperAdmin(mask)) {
      return false;
   }
   else {
      TiXmlElement elem("admin");
      elem.SetAttribute("mask",Tools::to_lower(mask));
      elem.SetAttribute("temp","0");
      this->root->FirstChild("super_admins")->InsertEndChild(elem) ;
      this->doc->SaveFile();
      return true;
   }
}

/**
 * Add a temporary super admin for the bot
 * @param mask Super admin's mask
 * @param duration Super admin time (in seconds)
 * @return True if the admin has been added, else false
 */
bool Admin::addTempSuperAdmin(string mask,unsigned int duration)
{
   time_t now;
	time(&now);
   if(this->isSuperAdmin(mask)) {
      return false;
   }
   else {
      TiXmlElement elem("admin");
      elem.SetAttribute("mask",Tools::to_lower(mask));
      elem.SetAttribute("temp","1");
      elem.SetAttribute("timeout",duration+now);
      this->root->FirstChild("super_admins")->InsertEndChild(elem) ;
      this->doc->SaveFile();
      return true;
   }
}

/** 
 * Del a super admin 
 * @param index Super admin index
 * @return True is the users has been deleted, else false
 */
bool Admin::delSuperAdmin(unsigned int index)
{
   bool result;
   TiXmlHandle docHandle (this->doc);
   TiXmlNode * parent ;
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_access").FirstChild("super_admins").Child(index).Element();
   if(elem) {
      parent = elem->Parent();
      result = parent->RemoveChild(elem);
      this->doc->SaveFile();
      return result;
   }
   return false;
}

/**
 * Tell if a user is a super admin
 * @param mask mask for witch you want to check the access
 * @return true if the user is a super admin, else false
 */
bool Admin::isSuperAdmin(string mask)
{
   TiXmlElement * elem;
   elem = this->root->FirstChild("super_admins")->FirstChildElement();
   while ( elem != NULL )
   {
      if(Tools::ircMaskMatch(Tools::to_lower(mask),Tools::to_lower(elem->Attribute("mask")) ))
         return true;
      elem = elem->NextSiblingElement();
   }
   return false;
}

/**
 * Tell is a mask is a super admin
 * @param mask Mask to test
 * @return true is it's a superadmin else false
 */
bool Admin::maskIsSuperAdmin(string mask)
{
   TiXmlElement * elem;
   string admMask;
   mask = Tools::to_lower(mask);
   string nick1,nick2,ident1,ident2,host1,host2;
   nick1 = mask.substr(0,mask.find("!"));
   ident1 = mask.substr(mask.find("!")+1,mask.find("@")-mask.find("!")-1); 
   host1 = mask.substr(mask.find("@")+1) ;
   elem = this->root->FirstChild("super_admins")->FirstChildElement();
   while ( elem != NULL )
   {
      admMask = Tools::to_lower(elem->Attribute("mask")) ;
      nick2 = admMask.substr(0,admMask.find("!"));
      ident2 = admMask.substr(admMask.find("!")+1,admMask.find("@")-admMask.find("!")-1);
      host2 = admMask.substr(admMask.find("@")+1) ;
      if((Tools::masksMatch((char*)nick1.c_str(),(char*)nick2.c_str())==1)&&(Tools::masksMatch((char*)ident1.c_str(),(char*)ident2.c_str())==1)&&(Tools::masksMatch((char*)host1.c_str(),(char*)host2.c_str())==1))
         return true;
      elem = elem->NextSiblingElement();
   }
   return false;
}

/**
 * Return the list of the super admins
 * @return a vector of string containing super admins' hosts
 */
vector<string> Admin::superAdminList()
{
   time_t futur;
	char date[17];
   vector<string> back;
   unsigned int i = 0 ;
   back.clear();
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_access").FirstChild("super_admins").Element();
   if (elem) {
      for (elem=elem->FirstChildElement() ; elem!=0;elem=elem->NextSiblingElement() ) {
         if ( (string)elem->Attribute("temp") == "1" ) {
            futur = Tools::strToInt(elem->Attribute("timeout")) ;
            strftime(date,18,"%y-%m-%d %X",localtime(&futur));
            back.push_back("#"+Tools::intToStr(i)+":"+elem->Attribute("mask")+" (to "+(string)date+")");
         }
         else {
            back.push_back("#"+Tools::intToStr(i)+":"+elem->Attribute("mask"));
         }
         i++;
      }
   }
   return back;
}

/**
 * Return Clear outdated temporary super admins
 */
void Admin::clearTempAdmins()
{
   time_t now;
   time(&now);
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_access").FirstChild("super_admins").Element();
   if (elem) {
      for (elem=elem->FirstChildElement() ; elem!=0;elem=elem->NextSiblingElement() ) {
         if ( (string)elem->Attribute("temp")=="1" ) {
            if ( Tools::strToInt(elem->Attribute("timeout"))<now ) {
               elem->Parent()->RemoveChild(elem);
            }
         }
      }   
      this->doc->SaveFile();
   }
}

/**
 * Add a user to a channel
 * @param channel Channel where to add the user
 * @param mask User's mask
 * @param level User's level
 * @return True if the user has benn added, else false
 */
bool Admin::addUser(string channel,string mask,unsigned int level)
{
   channel = Tools::to_lower(channel);
   mask = Tools::to_lower(mask);
   TiXmlElement * elem;
   if ( (this->userExists(channel,mask)) || (level<1) || (level>4) )
   {
      return false;
   }
   else
   {
      if (!this->channelExists(channel) )
      {
         this->addChannel(channel);
      }
      elem = this->root->FirstChild("channels")->FirstChildElement();
      while ( elem != NULL )
      {
         if ( Tools::to_lower(elem->Attribute("name")) == channel )
         {
            TiXmlElement elem2("user");
            elem2.SetAttribute("mask",mask);
            elem2.SetAttribute("level",level);
            elem->InsertEndChild(elem2) ;
            this->doc->SaveFile();
            return true;
         }
         elem = elem->NextSiblingElement();
      }
   }
   return false;
}

/**
 * Tell if a user exists for a channel
 * The user is recognized by a host
 * @param channel Channel where we want to make the test
 * @param mask User's mask
 * @return True if the user exists for the given channel, else false
 */
bool Admin::userExists(string channel,string mask)
{
   channel = Tools::to_lower(channel);
   mask = Tools::to_lower(mask);
   TiXmlElement * elem;
   TiXmlElement * elem2;
   elem = this->root->FirstChild("channels")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( Tools::to_lower(elem->Attribute("name")) == channel )
      {
         elem2 = elem->FirstChildElement();
         while ( elem2 != NULL )
         {
            if(Tools::to_lower(elem2->Attribute("mask")) == mask ) {
               return true;
            }
            elem2 = elem2->NextSiblingElement();
         }
         return false;
      }
      elem = elem->NextSiblingElement();
   }
   return false;
}

/**
 * Del a user from a channel
 * @param channel Channel where to del the user
 * @param mask User's mask to delete
 * @return True if the user has been added, else false
 */
bool Admin::delUser(string channel,string mask)
{
   channel = Tools::to_lower(channel);
   mask = Tools::to_lower(mask);
   bool back=false;
   TiXmlElement * elem;
   TiXmlElement * elem2;
   elem = this->root->FirstChild("channels")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( Tools::to_lower(elem->Attribute("name")) == channel )
      {
         elem2 = elem->FirstChildElement();
         while ( elem2 != NULL )
         {
            if(Tools::to_lower(elem2->Attribute("mask")) == mask ) {
               back = elem->RemoveChild(elem2);
               if (elem->FirstChild() == NULL )
               {
                  this->delChannel(channel);
               }
               this->doc->SaveFile();
               return back;
            }
            elem2 = elem2->NextSiblingElement();
         }
         return back;
      }
      elem = elem->NextSiblingElement();
   }
   return back;
}

/**
 * Update a User's level
 * @param channel user's channel
 * @param mask user's mask
 * @param level New user's level
 */
bool Admin::updateUserLevel(string channel,string mask,unsigned int level)
{
   channel = Tools::to_lower(channel);
   mask = Tools::to_lower(mask);
   if (level == 0 )
   {
      return this->delUser(channel,mask);
   }
   else if ( (level<1) || (level>4) )
   {
      return false;
   }
   TiXmlElement * elem;
   TiXmlElement * elem2;
   if (!this->userExists(channel,mask))
   {
      this->addUser(channel,mask,level);
      return true;
   }
   else
   {
      elem = this->root->FirstChild("channels")->FirstChildElement();
      while ( elem != NULL )
      {
         if ( Tools::to_lower(elem->Attribute("name")) == channel )
         {
            elem2 = elem->FirstChildElement();
            while ( elem2 != NULL )
            {
               if(Tools::to_lower(elem2->Attribute("mask")) == mask ) {
                  elem2->SetAttribute("level",level);
                  this->doc->SaveFile();
                  return true;
               }
               elem2 = elem2->NextSiblingElement();
            }
            return false;
         }
         elem = elem->NextSiblingElement();
      }   
   }
   return false;
}

/**
 * Get a user level for a channel
 * 0 for nothing
 * 1 for voice
 * 2 for op
 * 3 for master
 * 4 for owner
 * @param channel User's channel
 * @param mask User's mask
 * @return User's level (0 if non existing)
 */
unsigned int Admin::getUserLevel(string channel,string mask)
{
   channel = Tools::to_lower(channel);
   mask = Tools::to_lower(mask);
   TiXmlElement * elem;
   TiXmlElement * elem2;
   elem = this->root->FirstChild("channels")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( Tools::to_lower(elem->Attribute("name")) == channel )
      {
         elem2 = elem->FirstChildElement();
         while ( elem2 != NULL )
         {
            if(Tools::ircMaskMatch(mask,Tools::to_lower(elem2->Attribute("mask"))) ) {
               return Tools::strToInt(elem2->Attribute("level"));
            }
            elem2 = elem2->NextSiblingElement();
         }
         return 0;
      }
      elem = elem->NextSiblingElement();
   }
   return 0;
}

/**
 * Get mask level for a channel
 * 0 for nothing
 * 1 for voice
 * 2 for op
 * 3 for master
 * 4 for owner
 * @param channel User's channel
 * @param mask User's mask
 * @return User's level (0 if non existing)
 */
unsigned int Admin::getMaskLevel(string channel,string mask)
{
   channel = Tools::to_lower(channel);
   mask = Tools::to_lower(mask);
   TiXmlElement * elem;
   TiXmlElement * elem2;
   string admMask;
   string nick1,nick2,ident1,ident2,host1,host2;
   nick1 = mask.substr(0,mask.find("!"));
   ident1 = mask.substr(mask.find("!")+1,mask.find("@")-mask.find("!")-1); 
   host1 = mask.substr(mask.find("@")+1) ;
   elem = this->root->FirstChild("channels")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( Tools::to_lower(elem->Attribute("name")) == channel )
      {
         elem2 = elem->FirstChildElement();
         while ( elem2 != NULL )
         {
            admMask = Tools::to_lower(elem2->Attribute("mask")) ;
            nick2 = admMask.substr(0,admMask.find("!"));
            ident2 = admMask.substr(admMask.find("!")+1,admMask.find("@")-admMask.find("!")-1);
            host2 = admMask.substr(admMask.find("@")+1) ;
            if((Tools::masksMatch((char*)nick1.c_str(),(char*)nick2.c_str())==1)&&(Tools::masksMatch((char*)ident1.c_str(),(char*)ident2.c_str())==1)&&(Tools::masksMatch((char*)host1.c_str(),(char*)host2.c_str())==1)) {
               return Tools::strToInt(elem2->Attribute("level"));
            }
            elem2 = elem2->NextSiblingElement();
         }
         return 0;
      }
      elem = elem->NextSiblingElement();
   }
   return 0;
}

/**
 * Give all the access of a channel
 * @param channel Channel for the one you want the access list
 * @return A vector containing access for the given channel
 */
vector<string> Admin::chanLevels(string channel)
{
   vector<string> back;
   back.clear();
   TiXmlElement * elem;
   TiXmlElement * elem2;
   elem = this->root->FirstChild("channels")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( Tools::to_lower(elem->Attribute("name")) == Tools::to_lower(channel) )
      {
         elem2 = elem->FirstChildElement();
         while ( elem2 != NULL )
         {
            back.push_back(string(elem2->Attribute("mask"))+string(":")+string(elem2->Attribute("level")));
            elem2 = elem2->NextSiblingElement();
         }
         return back;
      }
      elem = elem->NextSiblingElement();
   }
   back.push_back(channel +" : unknown channel");
   return back;
}

/**
 * Return channel list
 * @return A vector containing 
 */
vector<string> Admin::getChannelsList()
{
   vector<string> back;
   back.clear();
   TiXmlElement * elem;
   elem = this->root->FirstChild("channels")->FirstChildElement();
   while ( elem != NULL )
   {
      back.push_back(elem->Attribute("name"));
      elem = elem->NextSiblingElement();
   }
   return back;
}

/**
 * Allow a command on a specific channel (disable for others)
 * @param command Onlyon command
 * @param channel Channel where to enable only the command
 * @return true if operation OK, else false
 */  
void Admin::addOnlyonCommand(string command,string channel)
{
   TiXmlElement elem("item");
   elem.SetAttribute("command",Tools::to_lower(command));
   elem.SetAttribute("channel",Tools::to_lower(channel));
   this->root->FirstChild("onlyon_commands")->InsertEndChild(elem) ;
   this->doc->SaveFile();
}

/**
 * Remove a Onlyon command
 * @param command Command on unonlyon
 * @param channel Channel where the command is no more onlyon
 */
void Admin::delOnlyonCommand(string command,string channel)
{
   TiXmlElement * elem;
   elem = this->root->FirstChild("onlyon_commands")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( (Tools::to_lower(elem->Attribute("command")) == Tools::to_lower(command)) && (Tools::to_lower(elem->Attribute("channel")) == Tools::to_lower(channel)) )
      {
         this->root->FirstChild("onlyon_commands")->RemoveChild(elem) ;
      }
      elem = elem->NextSiblingElement();
   }
   this->doc->SaveFile();
}

/** 
 * Enable a command on a channel
 * @param command Command to enable
 * @param channel Channel where to enable the command
 */
void Admin::enableCommand(string command,string channel)
{
   TiXmlElement * elem;
   elem = this->root->FirstChild("disabled_commands")->FirstChildElement();
   while ( elem != NULL )
   {
      if ( (Tools::to_lower(elem->Attribute("command")) == Tools::to_lower(command)) && (Tools::to_lower(elem->Attribute("channel")) == Tools::to_lower(channel)) )
      {
         this->root->FirstChild("disabled_commands")->RemoveChild(elem) ;
      }
      elem = elem->NextSiblingElement();
   }
   this->doc->SaveFile();
}

/** 
 * Disable a command on a channel
 * @param command Command to disable
 * @param channel Channel where to disable the command
 */
void Admin::disableCommand(string command,string channel)
{
   TiXmlElement elem("item");
   elem.SetAttribute("command",Tools::to_lower(command));
   elem.SetAttribute("channel",Tools::to_lower(channel));
   this->root->FirstChild("disabled_commands")->InsertEndChild(elem) ;
   this->doc->SaveFile();
}

/** 
 * Tell if a command can be executed on a channel
 * Check if the command is disabled or is not a "only on" one.
 * @param command Command to check
 * @param channel Channel to check
 * @return True if the command is ok, else false
 */
bool Admin::commandOK(string command,string channel)
{
   TiXmlElement * elem;
   vector<string> onlyon;
   elem = this->root->FirstChild("disabled_commands")->FirstChildElement();
   while ( elem != NULL )
   {
      if((Tools::to_lower(elem->Attribute("command")) == Tools::to_lower(command)) && (Tools::to_lower(elem->Attribute("channel")) == Tools::to_lower(channel)))
         return false;
      elem = elem->NextSiblingElement();
   }
   elem = this->root->FirstChild("onlyon_commands")->FirstChildElement();
   while ( elem != NULL )
   {
      if((Tools::to_lower(elem->Attribute("command")) == Tools::to_lower(command)) )
         onlyon.push_back(Tools::to_lower(elem->Attribute("channel")));
      elem = elem->NextSiblingElement();
   }
   if ( onlyon.size() > 0 )
   {
      if (! Tools::isInVector(onlyon,Tools::to_lower(channel)) )
         return false;
   }
   return true;
}

/** 
 * Give commands status (onlyon or disabled)
 * @return A vector containing commands status
 */
vector<string> Admin::commandsStatus()
{
   
   vector<string> back;
   back.clear();
   TiXmlElement * elem;
   elem = this->root->FirstChild("disabled_commands")->FirstChildElement();
   while ( elem != NULL )
   {  
      back.push_back(string(elem->Attribute("command"))+" disabled on "+string(elem->Attribute("channel")));
      elem = elem->NextSiblingElement();
   }
   elem = this->root->FirstChild("onlyon_commands")->FirstChildElement();
   while ( elem != NULL )
   {  
      back.push_back(string(elem->Attribute("command"))+" only on "+string(elem->Attribute("channel")));
      elem = elem->NextSiblingElement();
   }
   return back;
}

extern "C"
{ 
	Plugin *contruct_admin(BotKernel*b)
	{
		return new Admin(b);
	}
	void destroy_admin(Plugin*p)
	{
		delete p;
	}
	bool addsuperadmin (Message*m,Plugin*p,BotKernel*b)
	{
      Admin * adm = (Admin*)p;
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6 )
         {
            if ( m->getPart(4) == cf->getValue(p->getName()+".sapass"))
            {
               if (adm->addSuperAdmin(m->getPart(5)))
               {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(5) + " added to super admins list"));
                  b->getSysLog()->log(m->getPart(5) + " added to super admins list by"+ m->getSender(),INFO);
               }
            }
         }
      }
		return true;
	}
	bool addtempsuperadmin (Message*m,Plugin*p,BotKernel*b)
	{
      Admin * adm = (Admin*)p;
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 7 )
         {
            if ( m->getPart(4) == cf->getValue(p->getName()+".sapass"))
            {
               if (adm->addTempSuperAdmin(m->getPart(5),Tools::strtimeToSeconds(m->getPart(6))))
               {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(5) + " temporary added to super admins list ("+m->getPart(6)+")"));
                  b->getSysLog()->log(m->getPart(5) + " temporary added to super admins list by "+ m->getSender()+" ("+m->getPart(6)+")",INFO);
               }
            }
         }
      }
		return true;
	}
   bool setSuperAdminPass(Message*m,Plugin*p,BotKernel*b) 
   {
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6 )
         {
            if ( m->getPart(4) == cf->getValue(p->getName()+".sapass"))
            {
               cf->setValue(p->getName()+".sapass",m->getPart(5));
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Super admin pass changed to "+m->getPart(5)));
               b->getSysLog()->log("Super admin pass changed from "+ m->getPart(4) +" to "+m->getPart(5)+ " by "+m->getSender(),WARNING);
            }
         }
      }
      return true;
   }
   bool clearTemporaryAdmins(Message*m,Plugin*p,BotKernel*b) 
   {
      ((Admin*)p)->clearTempAdmins() ;
      return true;
   }
	bool superadminlist (Message*m,Plugin*p,BotKernel*b)
	{
      Admin * adm = (Admin*)p;
      if(m->isPrivate())
      {
         if ( adm->isSuperAdmin( m->getSender()))
            b->send(IRCProtocol::sendNotices(m->getNickSender(),Tools::gatherVectorElements(adm->superAdminList()," ",4)));
      }
		return true;
	}
	bool delsuperadmin (Message*m,Plugin*p,BotKernel*b)
	{
      Admin * adm = (Admin*)p;
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6 )
         {
            if ( m->getPart(4) == cf->getValue(p->getName()+".sapass"))
            {
               if (adm->delSuperAdmin(Tools::strToInt(m->getPart(5))))
               {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),"#"+m->getPart(5) + " removed from super admins list"));
                  b->getSysLog()->log("#"+m->getPart(5) + " removed from super admins list by "+m->getSender(),INFO);
               }
            }
         }
      }
		return true;
	}
   bool reset (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( adm->isSuperAdmin( m->getSender())) {
				b->getSysLog()->log("Bot reseted by "+m->getSender(),WARNING);
				b->setConnected(false);
         }
      }
		return true;
   }
   bool disconnect (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( adm->isSuperAdmin( m->getSender())) {
				b->getSysLog()->log("Bot stoped by "+m->getSender(),WARNING);
            b->send(IRCProtocol::quitServer("o/"));
            sleep(1);
			   b->stop();
         }
      }
		return true;
   }
   bool getconfvalue (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( (m->getSplit().size() == 5)&&(adm->isSuperAdmin( m->getSender())) )
         {
            b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" : "+cf->getValue(m->getPart(4),false)));
         }
      }
		return true;
   }
   bool setconfvalue (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6 )
         {
            if ( adm->isSuperAdmin( m->getSender())&& (m->getPart(4)!=(p->getName()+".sapass")))
            {
               cf->setValue(m->getPart(4),m->getPart(5));
               b->getSysLog()->log(m->getPart(4)+" set to " +m->getPart(5)+" by "+m->getSender(),INFO);
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" set to " +m->getPart(5)));
            }
         }
      }
		return true;
   }
   bool deletekey (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 5 )
         {
            if ( adm->isSuperAdmin( m->getSender())&& (m->getPart(4)!=(p->getName()+".sapass")))
            {
               if (cf->delKey(m->getPart(4)))
               {
                  b->getSysLog()->log(m->getPart(4)+" deleted by "+m->getSender(),INFO);
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" deleted"));
               }
               else
               {
                  b->send(IRCProtocol::sendNotice(m->getNickSender(),"key not found"));
               }
            }
         }
      }
		return true;
   }
   bool loadconffile (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( adm->isSuperAdmin( m->getSender()))
         {
            if(cf->load())
            {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Configuration file loaded"));
               b->getSysLog()->log("Configuration file loaded by "+m->getSender(),INFO);  
            }
            else
            {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Failed to load configuration file"));
               b->getSysLog()->log("Failed to load configuration file (by "+m->getSender()+")",WARNING); 
            }
         }
      }
		return true;
   }
   bool flushconffile (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( adm->isSuperAdmin( m->getSender()))
         {
            if(cf->flush())
            {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Configuration file flushed"));
               b->getSysLog()->log("Configuration file flushed by "+m->getSender(),INFO);
            }
            else
            {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Configuration file flushing failed"));
               b->getSysLog()->log("Configuration file flushing failed (by "+m->getSender()+")",WARNING);
            }
         }
      }
		return true;
   }
   bool setloglevel (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate() && (m->nbParts() == 5) && (adm->isSuperAdmin(m->getSender())) && ((m->getPart(4)=="error") ||(m->getPart(4)=="warning") || (m->getPart(4)=="info") || (m->getPart(4)=="nothing")))
      { 
         cf->setValue("kernel.logminlevel",m->getPart(4));
         b->getSysLog()->log("kernel.logminlevel set to " +m->getPart(4)+" by "+m->getSender(),INFO);
         b->getSysLog()->setLogLevel(m->getPart(4));
         b->send(IRCProtocol::sendNotice(m->getNickSender(),"kernel.logminlevel set to " +m->getPart(4)));
      }
      return true;
   }
   bool setlogkeepfiles (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate() && (m->nbParts() == 5) && (adm->isSuperAdmin(m->getSender())))
      { 
         cf->setValue("kernel.logkeepfiles",m->getPart(4));
         b->getSysLog()->log("kernel.logkeepfiles set to " +m->getPart(4)+" by "+m->getSender(),INFO);
         b->getSysLog()->setKeepFiles((m->getPart(4)=="1")?true:false);
         b->send(IRCProtocol::sendNotice(m->getNickSender(),"kernel.logkeepfiles set to " +m->getPart(4)));
      }
      return true;
   }
   bool setlogperiod (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
		ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate() && (m->nbParts() == 5) && (adm->isSuperAdmin(m->getSender())))
      { 
         cf->setValue("kernel.logperiod",m->getPart(4));
         b->getSysLog()->log("kernel.logperiod set to " +m->getPart(4)+" by "+m->getSender(),INFO);
         b->getSysLog()->setPeriodFormat(m->getPart(4));
         b->send(IRCProtocol::sendNotice(m->getNickSender(),"kernel.logperiod set to " +m->getPart(4)));
      }
      return true;
   }
   bool addOnlyon (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               adm->addOnlyonCommand(m->getPart(4),m->getPart(5));
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" only on "+m->getPart(5)+" by "+m->getSender()));
               b->getSysLog()->log(m->getPart(4)+" only on "+m->getPart(5)+" by "+m->getSender(),INFO);
            }
         }
      }
		return true;
   }
   bool delOnlyon (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               adm->delOnlyonCommand(m->getPart(4),m->getPart(5));
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" no more only on "+m->getPart(5)+" by "+m->getSender()));
               b->getSysLog()->log(m->getPart(4)+" no more only on "+m->getPart(5)+" by "+m->getSender(),INFO);
            }
         }
      }
		return true;
   }
   bool enable (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               adm->enableCommand(m->getPart(4),m->getPart(5));
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" enabled on "+m->getPart(5)+" by "+m->getSender()));
               b->getSysLog()->log(m->getPart(4)+" enabled on "+m->getPart(5)+" by "+m->getSender(),INFO);
            }
         }
      }
		return true;
   }
   bool disable (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 6)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               adm->disableCommand(m->getPart(4),m->getPart(5));
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" disabled on "+m->getPart(5)+" by "+m->getSender()));
               b->getSysLog()->log(m->getPart(4)+" disabled on "+m->getPart(5)+" by "+m->getSender(),INFO);
            }
         }
      }
		return true;
   }
	bool commandsStatus (Message*m,Plugin*p,BotKernel*b)
	{
      Admin * adm = (Admin*)p;
      if(m->isPrivate())
      {
         if ( adm->isSuperAdmin( m->getSender()))
            b->send(IRCProtocol::sendNotices(m->getNickSender(),Tools::gatherVectorElements(adm->commandsStatus()," | ",2)));
      }
		return true;
	}
	bool allowedCommandCheck (Message*m,Plugin*p,BotKernel*b)
	{
      Admin * adm = (Admin*)p;
  		ConfigurationFile * cf = b->getCONFF();
	   if ( (m->getSplit().size() >= 4) && (m->isPublic()) )
	   {
			if (m->getPart(3).length() > (":"+cf->getValue("kernel.command_prefix")).length())
         {
            if (! adm->commandOK(m->getPart(3).substr((":"+cf->getValue("kernel.command_prefix")).length()),m->getSource()) )
		         return false;
            else
               return true;
         }
         return true;
		}
      return true;
	}
	bool chanlev (Message*m,Plugin*p,BotKernel*b)
	{
      Admin * adm = (Admin*)p;
      if ( m->isPrivate() )
      {
         if ( m->getSplit().size() == 5)
         {
            if ( (adm->getUserLevel(m->getPart(4),m->getSender())>0) || (adm->isSuperAdmin(m->getSender())) )
            {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"Chanlev for "+m->getPart(4)+":"));
               b->send(IRCProtocol::sendNotices(m->getNickSender(),Tools::gatherVectorElements(adm->chanLevels(m->getPart(4))," ",3)));
            }
         }
         else if ( m->getSplit().size() == 7 )
         {  
            if ( (adm->getUserLevel(m->getPart(4),m->getSender())>2) || (adm->isSuperAdmin(m->getSender())) )
            {
               adm->updateUserLevel(m->getPart(4),m->getPart(5),Tools::strToInt(m->getPart(6)));
               b->send(IRCProtocol::sendNotice(m->getNickSender(),"done."));
            }  
         }
      }
      return true;
	}
   bool joinChannel (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 5)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               b->send(IRCProtocol::joinChannel(m->getPart(4)));
               b->getSysLog()->log("Joined "+m->getPart(4)+" (by "+m->getSender()+")",INFO);
            }
         }
      }
		return true;
   }
   bool leaveChannel (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() >= 5)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               b->send(IRCProtocol::leaveChannel(m->getPart(4),Tools::vectorToString(m->getSplit()," ",5)));
               b->getSysLog()->log("Left "+m->getPart(4)+" (by "+m->getSender()+")",INFO);
            }
         }
      }
		return true;
   }
   bool cycleChannel (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() >= 5)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               b->send(IRCProtocol::leaveChannel(m->getPart(4),"brb o/"));
               b->send(IRCProtocol::joinChannel(m->getPart(4)));
            }
         }
      }
		return true;
   }
   bool setNick (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
      ConfigurationFile * cf = b->getCONFF();
      if(m->isPrivate())
      {
         if ( m->getSplit().size() == 5)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               cf->setValue("kernel.nick",m->getPart(4));
               b->send( IRCProtocol::changeNick(m->getPart(4)) ) ;
               b->getSysLog()->log("Nick changed to "+m->getPart(4)+" (by "+m->getSender()+")",INFO);
               b->setNick(m->getPart(4));
            }
         }
      }
		return true;
   }
   bool tell (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() > 5)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               b->send(IRCProtocol::sendMsg(m->getPart(4),Tools::vectorToString(m->getSplit()," ",5)));
            }
         }
      }
		return true;
   }
   bool notice (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() > 5)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               b->send(IRCProtocol::sendNotice(m->getPart(4),Tools::vectorToString(m->getSplit()," ",5)));
            }
         }
      }
		return true;
   }
   bool raw (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p;     
      if(m->isPrivate())
      {
         if ( m->getSplit().size() > 5)
         {
            if ( adm->isSuperAdmin( m->getSender()))
            {
               b->send(Tools::vectorToString(m->getSplit()," ",4));
            }
         }
      }
		return true;
   }
   bool whoami (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string> msgs;
      vector<string> channs;
      unsigned int level = 0 ;
      msgs.push_back(m->getSender()+" :");
      Admin * adm = (Admin*)p;
      if ( adm->isSuperAdmin(m->getSender()))
         msgs.push_back("Bot Super Admin");
      channs = adm->getChannelsList() ;
      for ( unsigned int i = 0 ; i < channs.size() ; i ++ )
      {
         level = adm->getUserLevel(channs[i],m->getSender() ) ;
         if ( level > 0 )
            msgs.push_back(channs[i] + " : level " + Tools::intToStr(level) );
      }
      msgs.push_back("End.");
      b->send(IRCProtocol::sendNotices(m->getNickSender(),msgs)) ;
		return true;
   }
   bool reauth (Message*m,Plugin*p,BotKernel*b)
   {
      string buffer;
      Admin * adm = (Admin*)p; 
      plugin_function function ;
      pPlugin * pp = b->getPlugin("postconnect");    
      if(m->isPrivate())
      {
         if ( pp != NULL )
         {
            if (adm->isSuperAdmin( m->getSender()))
            { 
               function = (plugin_function)(intptr_t) dlsym(pp->handle,"onEndOfMOTD"); //*(void **)(&function) = dlsym(pp->handle,"onEndOfMOTD");
               if ( function != NULL )
               {
                  function(m,pp->object,b);
               }
            }
         }
      }
		return true;
   }
   bool onInvite (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
      if ( adm->isSuperAdmin( m->getSender()))
      if(m->isPrivate())
      {
         if ( adm->isSuperAdmin( m->getSender())) {
            b->send(IRCProtocol::joinChannel(m->getPart(3)));
				b->getSysLog()->log("Invited on "+ m->getSource()+" by "+m->getSender(),INFO);
         }
      }
		return true;
   }
   bool error (Message*m,Plugin*p,BotKernel*b)
   {
      b->getSysLog()->log(m->getMessage(),ERROR);
		return true;
   }
   bool getnbcountdowns (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
      if ( adm->isSuperAdmin( m->getSender()) && m->isPrivate()) {
         b->send(IRCProtocol::sendNotice(m->getNickSender(),"Countdowns : "+Tools::intToStr(b->getCountDowns()->size())));
      }
		return true;
   }
   bool clearCountDowns (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm = (Admin*)p;     
      if ( adm->isSuperAdmin( m->getSender()) && m->isPrivate()) {
         b->getCountDowns()->clear();
         b->send(IRCProtocol::sendNotice(m->getNickSender(),"Countdowns cleared"));
         b->getSysLog()->log("countdowns cleared by "+m->getSender(),INFO);
      }
		return true;
   }
}
