/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file admin.h
 * @brief Admin header file
 */

#ifndef ADMIN_H
#define ADMIN_H
#include "../plugin.h"
#include "../botkernel.h"

#ifndef HAVE_TINYXML
#include "../tinyxml/tinyxml.h"
#else 
#include <tinyxml.h>
#endif

#include "postconnect.h"
#include <iostream>
using namespace std;

/** @brief Bot access management
 *
 * This plugin stores (with an xml file) bot accounts (by host)
 * and provides stuff for administration
 */

class Admin : public Plugin
{
   private:
      /// Represent the xml document
      TiXmlDocument * doc ;
      /// Represent documents's root
      TiXmlNode* root;
      /// Initialize the XML file
      void initFile();
   public:
      /// Constructor
	   Admin(BotKernel*);
      /// Add a channel managed by the bot
      bool addChannel(string);
      /// Remove a channel managed by the bot
      bool delChannel(string);
      /// Tell if a channel is registred
      bool channelExists(string);
      /// Return channel list
      vector<string> getChannelsList();
      /// Add a super admin 
      bool addSuperAdmin(string);
      /// Add a temporary super admin
      bool addTempSuperAdmin(string,unsigned int);
      /// Clear timed out temporary super admins
      void clearTempAdmins();
      /// Del a super admin
      bool delSuperAdmin(unsigned int);
      /// Tell if a user is a super admin
      bool isSuperAdmin(string);
      /// Tell if a mask is super admin
      bool maskIsSuperAdmin(string);
      /// Give the super admin hosts list
      vector<string> superAdminList();
      /// Add a user to a channel
      bool addUser(string,string,unsigned int);
      /// Del a user from a channel
      bool delUser(string,string);
      /// Tell if a user exists for a channel
      bool userExists(string,string);
      /// Update a user level
      bool updateUserLevel(string,string,unsigned int);
      /// Get a user level for a channel
      unsigned int getUserLevel(string,string);
      /// Get mask level for a channel
      unsigned int getMaskLevel(string,string);
      /// Give all the access for a channel
      vector<string> chanLevels(string);
      /// Enable a command on a channel
      void enableCommand(string,string);
      /// Disable a command on a channel
      void disableCommand(string,string);
      /// Allow a command on a specific channel (disable for others)
      void addOnlyonCommand(string,string);
      /// Remove a Onlyon command
      void delOnlyonCommand(string,string);
      /// Tell if a command is can be executed on a channel
      bool commandOK(string,string);
      /// Give commands status (onlyon or disabled)
      vector<string> commandsStatus();
};


#endif

