/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file advertising.h
 * @brief Advertising header file
 */

#ifndef ADVERTISING_H
#define ADVERTISING_H
#include "../plugin.h"
#include "../botkernel.h"

#ifndef HAVE_TINYXML
#include "../tinyxml/tinyxml.h"
#else 
#include <tinyxml.h>
#endif


#include "admin.h"
#include <time.h>
#include <iostream>
using namespace std;

/** @brief Plugin managing ads
 *
 * This plugin makes the bot display ads on channels
 */

class Advertising : public Plugin
{
private:
   /// Represent the xml document
   TiXmlDocument * doc ;
   /// Represent documents's root
   TiXmlNode* root;
   /// Initialize the XML file
   void initFile();
public:
   /// Constructor
	Advertising(BotKernel*);
   /// Add an advertise to the advertise list
   time_t addAdvertise(string,unsigned int,unsigned int,string,string);
   /// Delete an advertise from the file
   bool delAdvertise(string);
   /// Tell if an advertise exists
   bool adExists(string);
   /// Get advertise infos
   vector<string> getAdvertiseInfos(string);
   /// Get advertises list
   vector<string> getAdvertisesList();
   /// Delete outdated ads
   void deleteOutdatedAds();
   /// launch an add
   void launchAdvertise(BotKernel*,string,unsigned int);
};


#endif

