/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file antiflood.h
 * @brief AntiFlood header file
 */

#ifndef ANTIFLOOD_H
#define ANTIFLOOD_H
#include "../plugin.h"
#include "../botkernel.h"
#include "admin.h"
#include <iostream>
using namespace std;

/** @brief Plugin that able the bot to detect flood
 *
 * trustyRC use an anti-excess-flood system build in the kernel that controls
 * output messages flow to make sure that the bot won't be killed by the server 
 * for "excess flood". This system make that the bot can sometimes take some time
 * to answer. The main case is when a lot of users use bot's commands : the kernel 
 * will store answers and people will think that the bot lags. It can be considered
 * as a flood attack from users. To avoid this problem, this plugin will watch messages' 
 * timestamps and if a message to treat is too old, it will drop it. Of course, only
 * PRIVMSG are concerned, to don't drop a "PING" command for example.
 * A configuration parameters exists to don't drop messages if they come from a super admin,
 * even if there timestamp is too old.
 */

class AntiFlood : public Plugin
{
private:
public:
   /// Constructor
	AntiFlood(BotKernel*);
};


#endif

