/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file danstonchat.cpp
 * @brief DansTonChat implementation file
 */

#include "danstonchat.h"

/**
 * Constructor
 */ 
DansTonChat::DansTonChat(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Display quotes from danstonchat.com";
	this->version = VERSION;
	this->name = "danstonchat";
   this->bindFunction("danstonchat",IN_COMMAND_HANDLER,"danstonchat",0,10);
   this->bindFunction("bashfr",IN_COMMAND_HANDLER,"danstonchat",0,10);
}

extern "C"
{
	Plugin *contruct_danstonchat(BotKernel*b)
	{
		return new DansTonChat(b);
	}
	void destroy_danstonchat(Plugin*p)
	{
		delete p;
	}
	bool danstonchat(Message*m,Plugin*p,BotKernel*b)
	{
      string QUOTE_BEGIN = "<p class=\"item item1\">";
      string QUOTE_TEXT_BEGIN = "</span><br />";
      string QUOTE_TEXT_END = "</p>";
      string ADSENSE = "</div><div class=\"adsense\">";
      string buffer;
      string::size_type pos ;
      Socket sock = Socket();
      string query;
      if (m->isPublic()) 
      {
         if (!sock.connectSock(80,"danstonchat.com","")) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Unable to connect to danstonchat.com *"));
            return true;
         }
         if (m->nbParts() == 5) {
            query = "GET /?"+m->getPart(4)+" HTTP/1.1\nHost: danstonchat.com\n\n";
         }
         else if (m->nbParts() == 4 ) {
            query = "GET /?sort=random HTTP/1.1\nHost: danstonchat.com\n\n" ;
         }
         else {
            return true;
         }
         sock.sendStr(query) ;
         while ((buffer.find(ADSENSE)==string::npos)&&(buffer.find("</body>")==string::npos)) {
            buffer += sock.receive();
         }
         pos = buffer.find(QUOTE_BEGIN);
         if ( pos == string::npos) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Quote non trouvee *"));
            return true;
         }
         buffer = buffer.substr(pos+QUOTE_BEGIN.length()) ;
         pos = buffer.find(QUOTE_TEXT_BEGIN);
         if ( pos == string::npos) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Parse error *"));
            return true;
         }
         buffer = buffer.substr(pos+QUOTE_TEXT_BEGIN.length()) ;
         pos = buffer.find(QUOTE_TEXT_END);
         if ( pos == string::npos) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Parse error *"));
            return true;
         }
         buffer = buffer.substr(0,pos) ;
         vector<string> lines = Tools::stringToVector(buffer,"<br />\r\n",0);
         if ( lines.size() <= Tools::strToUnsignedInt(b->getCONFF()->getValue(p->getName()+".maxlines")) ) {
            for (unsigned int i = 0 ; i < lines.size() ; i ++ ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),Tools::clearAccents(Tools::cleanHTML(lines[i]))));
            }
         }
         else {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Too many lines. Sorry...*"));
         }
      }
		return true;
	}
}
