/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file fedorafr.cpp
 * @brief Fedorafr implementation file
 */

#include "fedorafr.h"

/**
 * Constructor
 */ 
Fedorafr::Fedorafr(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Plugins that allow people to search on Fedora-fr.org wiki and planet";
	this->version = VERSION;
	this->name = "fedorafr";
	this->bindFunction("wiki",IN_COMMAND_HANDLER,"wiki",0,10);
	this->bindFunction("planet",IN_COMMAND_HANDLER,"planet",0,10);
	this->bindFunction("paste",IN_COMMAND_HANDLER,"displayPaste",0,10);
}



/**
 * Extract links in a wiki result page
 * @param datas HTML code from wiki page
 * @return Wiki's URLs 
 */
vector<string> Fedorafr::getWikiLinks(string datas) {
   string result;
   vector<string> results;
   const string BASE_URL = "http://doc.fedora-fr.org" ;
   string::size_type pos;
   vector<string> split = Tools::stringToVector(datas,"\n",0) ;  
   for (unsigned int i = 0 ; i < split.size() ; i ++ ) {
      pos = split[i].find("href=\"");
      if ( pos != string::npos ) {
         result = BASE_URL+split[i].substr(pos+string("href=\"").size(),split[i].find(" title=\"")-(pos+string("href=\"").size()+1)) ;
         if ( result.find("Discuter:") == string::npos) {
            results.push_back(result);
         }
      }
   }
   return results;
}

extern "C"
{
	Plugin *contruct_fedorafr(BotKernel*b)
	{
		return new Fedorafr(b);
	}
	void destroy_fedorafr(Plugin*p)
	{
		delete p;
	}
	bool wiki (Message*m,Plugin*p,BotKernel*b)
	{
      const string NO_RESULT_TEXT = "Aucun texte d’article ne correspond à la recherche.";
      const string RESULTS = "mw-search-results";
      const string POWERSEARCH = "<form id=\"powersearch\"" ;
      const string END_TITLE = "</span></h2>";
      const string SEARCH_BOTTOM = "mw-search-pager-bottom" ;
      string buffer = "";
      string title = "";
      string::size_type pos=0;
      unsigned int startFrom=5;
      unsigned int nbResults ;
      vector<string> results;
      unsigned int maxResults = Tools::strToInt(b->getCONFF()->getValue(p->getName()+".wiki_max_results"));
      Fedorafr*ffr = (Fedorafr*)p;
      Socket sock = Socket();
      if ( m->isPublic() && (m->nbParts() >= 5) ) {
         if (!sock.connectSock(80,"doc.fedora-fr.org","")) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Unable to connect to doc.fedora-fr.org *"));
            return true;
         }
         nbResults = Tools::strToInt(m->getPart(4));
         if (nbResults == 0 ) {
            nbResults = Tools::strToInt(b->getCONFF()->getValue(p->getName()+".wiki_default_results"));  
            startFrom = 4;
         }
         else  if (nbResults > maxResults ) {
            nbResults = maxResults ;
         }
         sock.sendStr("GET /wiki/Special:Search?search="+Tools::urlencode(Tools::vectorToString(m->getSplit()," ",startFrom) )+"&fulltext=Rechercher HTTP/1.1\nHost: doc.fedora-fr.org\n\n");
         while (buffer.find(POWERSEARCH)==string::npos) {
            buffer += sock.receive() ;
         }
         pos = buffer.find(RESULTS);
         if ( pos == string::npos) {
	         b->send(IRCProtocol::sendMsg(m->getSource(),"Aucun resultat."));
            return true;
         }
         buffer = buffer.substr(pos+string(RESULTS).length()+2);
         pos = buffer.find(SEARCH_BOTTOM);
         if ( pos == string::npos) {
	         b->send(IRCProtocol::sendMsg(m->getSource(),"* Error while parsing result * (1)"));
            return 0;
         }
         results = ffr->getWikiLinks(buffer.substr(0,pos)) ;
         if (nbResults > results.size() ) {
            nbResults = results.size();
         } 
         for ( unsigned int i = 0 ; i  < nbResults ; i ++ ) {
            b->send(IRCProtocol::sendMsg(m->getSource(),results[i]));
         }
      }
		return true;
	}
	bool planet (Message*m,Plugin*p,BotKernel*b)
	{
      const string BEGIN_HREF = "<h2><a href=\"";
      const string END_HREF = "\" title=";
      string buffer = "";
      string::size_type pos=0;
      unsigned int startFrom=5;
      unsigned int nbResults ;
      vector<string> results;
      unsigned int maxResults = Tools::strToInt(b->getCONFF()->getValue(p->getName()+".planet_max_results"));
      Socket sock = Socket();
      if ( m->isPublic() && (m->nbParts() >= 5) ) {
         if (!sock.connectSock(80,"planet.fedora-fr.org","")) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Unable to connect to planet.fedora-fr.org *"));
            return true;
         }
         nbResults = Tools::strToInt(m->getPart(4));
         if (nbResults == 0 ) {
            nbResults = Tools::strToInt(b->getCONFF()->getValue(p->getName()+".planet_default_results"));  
            startFrom = 4;
         }
         else  if (nbResults > maxResults ) {
            nbResults = maxResults ;
         }
         sock.sendStr("GET content/advancedsearch?SearchPageLimit=2&SearchText="+Tools::urlencode(Tools::vectorToString(m->getSplit()," ",startFrom) )+" HTTP/1.1\nHost: planet.fedora-fr.org\n\n");
         while (buffer.find("</body>")==string::npos) {
            buffer += sock.receive() ;
         } 
         do {
            pos = buffer.find(BEGIN_HREF);
            if (pos!=string::npos) {
               buffer = buffer.substr(pos+BEGIN_HREF.length());
               results.push_back(buffer.substr(0,buffer.find(END_HREF) ));
            }
         }
         while (pos !=string::npos); 
         if ( results.size() > 0 ) {
            if ( results.size() < nbResults ) {
               nbResults = results.size() ;
            }
            for (unsigned int i = 0 ; i < nbResults ; i ++ ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),results[i]));
            }
         }
         else {
            b->send(IRCProtocol::sendMsg(m->getSource(),"Aucun resultat"));
         }
      }
		return true;
	}
	bool displayPaste (Message*m,Plugin*p,BotKernel*b)
	{
      if (m->isPublic() )
         b->send(IRCProtocol::sendMsg(m->getSource(),"http://fedora-fr.pastebin.com ou http://rafb.net/paste/"));
      return true;
   }
}
