/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file gameserver.h
 * @brief GameServer header file
 */

#ifndef GAMESERVER_H
#define GAMESERVER_H
#include "../plugin.h"
#include "../botkernel.h"
#include <iostream>
#include <map>
using namespace std;

/// Maximum chars for server's answer
const int MAX_CHARS=1000;

/** @brief Provides tools to query game servers
 *
 * Provides tools to query game servers
 */
class GameServer : public Plugin
{
private:
public:
   /// Constructor
	GameServer(BotKernel*);
   /// Convert a string to a long (seems fucked)
   long strToLong(string);
   /// Get a string in HL1 protocol
   string getHLstring(unsigned int * ,char*);
   /// Get a byte in HL1 protocol
   char getHLbyte(unsigned int*,char*);
   /// Get a long in HL1 protocol (as a string)
   string getHLlong(unsigned int*,char*);
   /// Get players list in HL1 protocol 
   bool getHL1Players(vector<string>*,char*);
   /// Get HL1 challenge
   bool getHL1Challenge(string*,char*);
   /// Get HL1 server's settings 
   bool getHL1Infos(map<string,string>* ,char*);
   /// Get a Q3 gametype according to a number 
   string getQ3GameType(string);
   /// Get warsow server's infos (settings and players) 
   bool parseWSWinfos(map<string,string>*,vector<string>*,char*);
   /// Get Q3 server's infos (settings and players) 
   bool parseQ3infos(map<string,string>*,vector<string>*,char*);
   /// Send a query to a server
   bool sendQuery(string,string,int*,string) ;
   /// Get a result from a server
   string getResult(int,char*) ;
};


#endif

