/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file ignore.h
 * @brief Ignore header file
 */

#ifndef IGNORE_H
#define IGNORE_H
#include "../plugin.h"
#include "../botkernel.h"

#ifndef HAVE_TINYXML
#include "../tinyxml/tinyxml.h"
#else 
#include <tinyxml.h>
#endif


#include "admin.h"
#include <iostream>
using namespace std;

/** @brief Manage ignores
 *
 * This class provides an ignore system that allow 
 * the bot to ignore users
 */

class Ignore : public Plugin
{
   private:
      /// Represent the xml document
      TiXmlDocument * doc ;
      /// Represent documents's root
      TiXmlNode* root;
      /// Initialize the XML file
      void initFile();
   public:
      /// Constructor
	   Ignore(BotKernel*);
      /// Add a host to the ignore list
		void addIgnore(string,string,unsigned int);
      /// Remove a host from ignore list
		bool delIgnore(unsigned int);
      /// Tell if a host is ignored
		bool isIgnored(string);
      /// Return the ignore list
		vector<string> getIgnoreList();
      /// Clear the XML file from out dated ignores
		void purifyList();
};


#endif

