/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file infos.cpp
 * @brief Infos implementation file
 */

#include "infos.h"

/**
 * Constructor
 */ 
Infos::Infos(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Infos about kernel";
	this->version = VERSION;
	this->name = "infos";
	this->bindFunction("uptime",IN_COMMAND_HANDLER,"uptime",0,10);
	this->bindFunction("version",IN_COMMAND_HANDLER,"version",0,10);
	this->bindFunction("sysinfos",IN_COMMAND_HANDLER,"sysinfos",0,10);
	this->bindFunction("online",IN_COMMAND_HANDLER,"online",0,10);
	this->bindFunction("!prefix",IN_FREE_COMMAND_HANDLER,"prefix",0,10);
	this->bindFunction("!help",IN_FREE_COMMAND_HANDLER,"help",0,10);
	this->bindFunction("help",IN_COMMAND_HANDLER,"help",0,10);
}

extern "C"
{
	Plugin *contruct_infos(BotKernel*b)
	{
		return new Infos(b);
	}
	void destroy_infos(Plugin*p)
	{
		delete p;
	}
	bool uptime(Message*m,Plugin*p,BotKernel*b)
	{
      time_t now;
      time(&now);
      double seconds,minutes,hours,days,weeks;
      seconds = difftime(now,b->getStartTime());
      minutes = seconds/60 ;
      hours = minutes /60;
      days = hours / 24 ;
      weeks = days / 7 ;
      seconds = int(seconds)%60;
      minutes = int(minutes)%60;
      hours = int(hours)%24;
      days = int(days)%7;
      weeks = int(weeks);
      b->send(IRCProtocol::sendNotice(m->getNickSender(),"* " + Tools::intToStr((int)weeks) + "w " + Tools::intToStr((int)days) + "d "  + Tools::intToStr((int)hours) + "h " +Tools::intToStr((int)minutes) + "m " +Tools::intToStr((int)seconds) + "s *"));  
		return true;
	}
	bool version(Message*m,Plugin*p,BotKernel*b)
	{
      b->send(IRCProtocol::sendNotice(m->getNickSender(),b->getVersion()));
		return true;
	}
	bool sysinfos(Message*m,Plugin*p,BotKernel*b)
	{
      if (system (((string)"uname -a > "+b->getDatasDir()+"sysinfo.log && uptime >> "+b->getDatasDir()+"sysinfo.log").c_str()) != -1 ) {
         std::ifstream fileinfos((b->getDatasDir()+"sysinfo.log").c_str() );
         if ( fileinfos ) {
            string line; 
            while ( getline( fileinfos, line ) ) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),line));
            }
            fileinfos.close();
         }
      }
      else {
         b->send(IRCProtocol::sendNotice(m->getNickSender(),"Execution error"));
      }
		return true;
	}
	bool online(Message*m,Plugin*p,BotKernel*b)
	{
      time_t now;
      time(&now);
      double seconds,minutes,hours,days,weeks;
      seconds = difftime(now,b->getStartOnline());
      minutes = seconds/60 ;
      hours = minutes /60;
      days = hours / 24 ;
      weeks = days / 7 ;
      seconds = int(seconds)%60;
      minutes = int(minutes)%60;
      hours = int(hours)%24;
      days = int(days)%7;
      weeks = int(weeks);
      b->send(IRCProtocol::sendNotice(m->getNickSender(),"* " + Tools::intToStr((int)weeks) + "w " + Tools::intToStr((int)days) + "d " + Tools::intToStr((int)hours) + "h " +Tools::intToStr((int)minutes) + "m " +Tools::intToStr((int)seconds) + "s *"));  
		return true;
	}
	bool prefix(Message*m,Plugin*p,BotKernel*b)
	{
      ConfigurationFile*cff = b->getCONFF();
      b->send( IRCProtocol::sendNotice(m->getNickSender(),"Prefix for command is : '"+cff->getValue("kernel.command_prefix"))+"'" ); 
		return true;
	}
	bool help (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
      b->send( IRCProtocol::sendNotice(m->getNickSender(),"Prefix for command is : '"+cff->getValue("kernel.command_prefix")+"' - Manual at "+cff->getValue(p->getName()+".helpurl"))) ;
      return false;
   }
}
