/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file lamoule.h
 * @brief Lamoule header file
 */

#ifndef LAMOULE_H
#define LAMOULE_H
#include "../plugin.h"
#include "../botkernel.h"

#ifndef HAVE_TINYXML
#include "../tinyxml/tinyxml.h"
#else 
#include <tinyxml.h>
#endif


#include "admin.h"
#include "usersinfos.h"
#include <iostream>
using namespace std;

/// Available sort criterions
typedef enum {
	TOTAL,AVERAGE
} sort_criterion;

/** @brief Manage lamoule's ladder
 *
 * Manage lamoule's ladder.
 * Score are stored in an XML file
 */

class Lamoule : public Plugin
{
private:
   /// Represent the xml document
   TiXmlDocument * doc ;
   /// Represent documents's root
   TiXmlNode* root;
   /// Initialize the XML file
   void initFile();
   /// Stores the next score
   unsigned int nextScore;
   /// Maximul score
   int MAX_SCORE;
   /// First floor
   int FIRST_FLOOR;
   /// Second floor
   int SECOND_FLOOR;
public:
   /// Constructor
	Lamoule(BotKernel*);
   /// Get the lamoule's topshot
   vector<string> getTopShot();
   /// Set the lamoule's topshot
   void setTopShot(string,string,string);
   /// Add a player in the ladder
   void addPlayer(string,unsigned int);
   /// Increase score for a player
   char increaseScore(string,int,unsigned int,bool);
   /// Sort elements to get stats
   vector<TiXmlElement *> sort(sort_criterion,int);
   /// Get 5 first 
   vector<string> get5first(sort_criterion,int);
   /// Get informations about a player
   vector<string> getInfosPlayer(string,int);
   /// Remove a player from the ladder
   bool deletePlayer(string);
   /// Purify File 
   void purifyFile(int);
   /// Set next score
   bool setNextScore(int);
   /// generate score
   int generateScore() ;
};


#endif

