/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file logfactory.h
 * @brief LogFactory header file
 */

#ifndef LOGFACTORY_H
#define LOGFACTORY_H
#include "../plugin.h"
#include "../botkernel.h"
#include "../channel.h"
#include "usersinfos.h"
#include <iostream>
#include <map>
using namespace std;

/** @brief This plugin manage channels logging
 *
 * This plugin manage channels logging. It uses LogFile class
 * provided by the kernel. LogFiles are stored in datas directory.
 */

class LogFactory : public Plugin
{
private:
   /// Map that sotred LogFile objects
   map<string,LogFile*>* logs;
   /// Stores a kernel pointerr
   BotKernel* kernel;
public:
   /// Constructor
	LogFactory(BotKernel*);
   /// Destructor
   ~LogFactory();
   /// Tell if a given channel has to be logged
   bool hasToBeLogged(string);
   /// Return logged channels
   vector<Channel*> getLoggedChannels();
   /// Destroy logs
   void destroyLogs();
   /// clean logs
   void cleanLogs();
   /// Open a new log for a channel
   bool newLog(string);
   /// Clode and delete a log
   void closeLog(string);
   /// Log an event in a LogFile object
   bool log(string,string);
};


#endif

