/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file magic8ball.cpp
 * @brief Magic8Ball implementation file
 */

#include "magic8ball.h"

/**
 * Constructor
 */ 
Magic8Ball::Magic8Ball(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "magic 8 ball game";
	this->version = VERSION;
	this->name = "magic8ball";
	this->bindFunction("8ball",IN_COMMAND_HANDLER,"ball",0,10);

   this->answers[0] = "As I see it, yes" ;
   this->answers[1] = "Ask again later" ;
   this->answers[2] = "Better not tell you now" ;
   this->answers[3] = "Cannot predict now" ;
   this->answers[4] = "Concentrate and ask again" ;
   this->answers[5] = "Don't count on it" ;
   this->answers[6] = "It is certain" ;
   this->answers[7] = "It is decidedly so" ;
   this->answers[8] = "Most likely" ;
   this->answers[9] = "My reply is no" ;
   this->answers[10] = "My sources say no" ;
   this->answers[11] = "Outlook good" ;
   this->answers[12] = "Outlook not so good" ;
   this->answers[13] = "Reply hazy, try again" ;
   this->answers[14] = "Signs point to yes" ;
   this->answers[15] = "Very doubtful" ;
   this->answers[16] = "Without a doubt" ;
   this->answers[17] = "Yes" ;
   this->answers[18] = "Yes - definitely" ;
   this->answers[19] = "You may rely on it" ;
}

/**
 * Get a random answer
 * @return the answer
 */
string Magic8Ball::getRandomAnswer() {
   return this->answers[Tools::random(0,19)];
}

extern "C"
{
	Plugin *contruct_magic8ball(BotKernel*b)
	{
		return new Magic8Ball(b);
	}
	void destroy_magic8ball(Plugin*p)
	{
		delete p;
	}
	bool ball (Message*m,Plugin*p,BotKernel*b)
	{
      Magic8Ball* mb = (Magic8Ball*) p;
      if (m->isPublic()&&(m->nbParts() >= 5)) {
         b->send( IRCProtocol::sendMsg(m->getSource(),mb->getRandomAnswer() ) );
      }
		return true;
	}
}
