/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file moderation.cpp
 * @brief Moderation implementation file
 */

#include "moderation.h"

/**
 * Constructor
 */ 
Moderation::Moderation(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "channels moderation";
	this->version = VERSION;
	this->name = "moderation";
	this->bindFunction("op",IN_COMMAND_HANDLER,"op",0,10);
	this->bindFunction("unop",IN_COMMAND_HANDLER,"unop",0,10);
	this->bindFunction("voice",IN_COMMAND_HANDLER,"voice",0,10);
	this->bindFunction("unvoice",IN_COMMAND_HANDLER,"unvoice",0,10);
	this->bindFunction("topic",IN_COMMAND_HANDLER,"topic",0,10);
	this->bindFunction("kick",IN_COMMAND_HANDLER,"kick",0,10);
	this->bindFunction("kickall",IN_COMMAND_HANDLER,"kickall",0,10);
	this->bindFunction("opall",IN_COMMAND_HANDLER,"opall",0,10);
	this->bindFunction("unopall",IN_COMMAND_HANDLER,"unopall",0,10);
	this->bindFunction("voiceall",IN_COMMAND_HANDLER,"voiceall",0,10);
	this->bindFunction("unvoiceall",IN_COMMAND_HANDLER,"unvoiceall",0,10);
	this->bindFunction("masskick",IN_COMMAND_HANDLER,"masskick",0,10);
	this->bindFunction("randomkick",IN_COMMAND_HANDLER,"randomKick",0,10);
	this->bindFunction("autoop",IN_COMMAND_HANDLER,"autoop",0,10);
	this->bindFunction("unautoop",IN_COMMAND_HANDLER,"unautoop",0,10);
	this->bindFunction("autovoice",IN_COMMAND_HANDLER,"autovoice",0,10);
	this->bindFunction("unautovoice",IN_COMMAND_HANDLER,"unautovoice",0,10);
	this->bindFunction("protecttopic",IN_COMMAND_HANDLER,"protecttopic",0,10);
	this->bindFunction("unprotecttopic",IN_COMMAND_HANDLER,"unprotecttopic",0,10);
	this->bindFunction("protectmodes",IN_COMMAND_HANDLER,"protectmodes",0,10);
	this->bindFunction("unprotectmodes",IN_COMMAND_HANDLER,"unprotectmodes",0,10);
	this->bindFunction("banmask",IN_COMMAND_HANDLER,"banmask",0,10);
	this->bindFunction("banlist",IN_COMMAND_HANDLER,"banlist",0,10);
	this->bindFunction("ban",IN_COMMAND_HANDLER,"ban",0,10);
	this->bindFunction("baninfos",IN_COMMAND_HANDLER,"baninfos",0,10);
	this->bindFunction("bandel",IN_COMMAND_HANDLER,"bandel",0,10);
	this->bindFunction("unbanall",IN_COMMAND_HANDLER,"unbanall",0,10);
	this->bindFunction("invite",IN_COMMAND_HANDLER,"invite",0,10);
	this->bindFunction("15",IN_LOOP,"clearOutBans",0,10);
	this->bindFunction("JOIN",IN_TYPE_HANDLER,"joinHandler",0,10);
	this->bindFunction("PART",IN_TYPE_HANDLER,"partHandler",0,10);
	this->bindFunction("QUIT",IN_TYPE_HANDLER,"quitHandler",0,10);
	this->bindFunction("KICK",IN_TYPE_HANDLER,"kickHandler",0,10);
	this->bindFunction("MODE",IN_TYPE_HANDLER,"modeHandler",0,10);
	this->bindFunction("MODE",IN_TYPE_HANDLER,"modeHandlerProtect",0,10);
	this->bindFunction("TOPIC",IN_TYPE_HANDLER,"topicHandler",0,10);
   this->bindFunction("474",IN_TYPE_HANDLER,"bannedHandler",0,10);
	this->bindFunction("332",IN_TYPE_HANDLER,"topicJoin",0,10);
   this->addRequirement("admin");
   this->addRequirement("usersinfos");

   this->doc = new TiXmlDocument(b->getDatasDir()+"moderation.xml");
   if ( this->doc->LoadFile() )
   {
      this->root = this->doc->FirstChild();
   }
   else
   {
      this->initFile();
   }
}

/**
 * Initilaize the XML file by creating root and first childs (file empty structure)
 */
void Moderation::initFile()
{
   TiXmlElement born("trustyrc_moderation");
   this->doc->InsertEndChild(born);
   this->root = this->doc->FirstChild();
   TiXmlElement bansNode("bans");
   this->root->InsertEndChild(bansNode);
   this->doc->SaveFile();
}

/**
 * Add a ban for a host on a given channel
 * Stores the ban in a XML file
 * @param channel Channel on witch the user is banned
 * @param mask User's mask : nick!ident\@host
 * @param duration ban time (seconds)
 * @param by banner mask
 * @param reason Reason for the ban
 * @return true if the user has been banned, else false
 */
bool Moderation::addBan(string channel,string mask,unsigned int duration,string by,string reason)
{
   if ( !this->isBanned(channel,mask) )
   {
      TiXmlElement * elemChan;
      TiXmlHandle docHandle (this->doc);
      TiXmlHandle bansHandle = docHandle.FirstChild("trustyrc_moderation").FirstChild("bans") ;
      elemChan = bansHandle.FirstChild(channel.substr(1)).Element() ;
      if(!elemChan) {
         TiXmlElement newChan(channel.substr(1));
         bansHandle.Element()->InsertEndChild(newChan);
         elemChan = bansHandle.FirstChild(channel.substr(1)).Element() ;
      }
      time_t now;
	   time(&now);
	   char date[17];
      TiXmlElement item("ban");
      item.SetAttribute("mask",mask);
      item.SetAttribute("timestamp",now);
	   strftime(date,18,"%y-%m-%d %X",localtime(&now));
   	item.SetAttribute("date",date);
      item.SetAttribute("duration",duration);
      item.SetAttribute("by",by);
      item.SetAttribute("reason",reason);
      elemChan->InsertEndChild(item);
      this->doc->SaveFile();
      return true;
   }
   return false;
} 

/** 
 * Tell if a user is banned on a ginven channel
 * @param channel Channel to test
 * @param mask User's mask
 * @return True is the user is banned, else false
 */ 
bool Moderation::isBanned(string channel,string mask)
{
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem;
   elem = docHandle.FirstChild("trustyrc_moderation").FirstChild("bans").FirstChild(channel.substr(1)).Element();
   if (elem) {
      for( elem=elem->FirstChildElement(); elem!=0; elem=elem->NextSiblingElement() ) {
		   if ( Tools::ircMaskMatch(mask,elem->Attribute("mask")) ) {
            return true;
         }
	   }
      return false;
   }
   return false;
}

/**
 * Delete a ban and return the mask (to delete it on the chan)
 * @param channel Channel where delete the ban
 * @param index Ban index
 * @return ban's mask deleted, or empty string if ban not found
 */
string Moderation::delBan(string channel,unsigned int index)
{
   string mask = "";
   TiXmlHandle docHandle (this->doc);
   TiXmlNode * parent ;
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_moderation").FirstChild("bans").FirstChild(channel.substr(1)).Child(index).Element() ;
   if ( elem ) {
      mask = elem->Attribute("mask");
      parent = elem->Parent();
      parent->RemoveChild(elem);
      if ( parent->FirstChild() == NULL ) {
         parent->Parent()->RemoveChild(parent);
      }
      this->doc->SaveFile();
      return mask;
   }
   return mask;
}

/**
 * Return ban list for a given channel
 * @param channel Channel to list
 * @return A vector containing ban list
 */
vector<string> Moderation::getBanList(string channel)
{
   unsigned int i = 0 ;
   vector<string> back;
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_moderation").FirstChild("bans").FirstChild(channel.substr(1)).Element();
   if (elem) {
      for( elem=elem->FirstChildElement(); elem!=0; elem=elem->NextSiblingElement() ) {
         back.push_back("#"+Tools::intToStr(i)+":"+elem->Attribute("mask") );
         i++;
	   }
   }
   else {
      back.push_back("No bans for "+channel);
   }
   return back;
}

/**
 * Return informations about a ban
 * @param channel Ban channel
 * @param index Ban index
 * @return A vector containing informations
 */
vector<string> Moderation::banInfos(string channel,unsigned int index)
{
	char date[17];
   time_t futur;
   string endBan = "";
   vector<string> result;
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem = docHandle.FirstChild("trustyrc_moderation").FirstChild("bans").FirstChild(channel.substr(1)).Child(index).Element() ;
   if ( elem ) {
      result.push_back(string(elem->Attribute("mask"))+" by " +string(elem->Attribute("by"))); 
      futur = Tools::strToInt(elem->Attribute("timestamp"))+Tools::strToInt(elem->Attribute("duration")) ;
      if ( futur > Tools::strToInt(elem->Attribute("timestamp")) ) {
         strftime(date,18,"%y-%m-%d %X",localtime(&futur));
         endBan = " to " +string(date) ;
      }
      else {
         endBan = " (permanent)" ;
      }
      result.push_back("on "+string(elem->Attribute("date"))+endBan+" : "+string(elem->Attribute("reason"))); 
   }
   else {
      result.push_back("nonexistent"); 
   }
   return(result);
}

/**
 * Clear all bans for a given channel
 * even if they are not out dated
 * Return them to be deleted on the channel
 * @param channel Channel where delete bans
 * @return A vector containing the deleted bans
 */
vector<string> Moderation::clearList(string channel)
{
   vector<string> deletedBans;
   deletedBans.clear();
   TiXmlNode * parent;
   TiXmlElement*chan,*elem;
   TiXmlHandle docHandle (this->doc);
   chan = docHandle.FirstChild("trustyrc_moderation").FirstChild("bans").FirstChild(channel.substr(1)).Element();
   if (chan) {
      parent = chan->Parent();
      for( elem=chan->FirstChildElement(); elem!=0; elem=elem->NextSiblingElement() ) {
         deletedBans.push_back(elem->Attribute("mask"));
	   }
      parent->RemoveChild(chan);
      this->doc->SaveFile();
   }
   return deletedBans;
}

/**
 * Clear all outdated bans 
 * Return them to be deleted on the channel
 * @param myChans Chans where the bot is (whithout '#' first char), to be sure to not delete bans where the bot is not present
 * @return A vector containing the deleted bans
 */
vector<string> Moderation::clearOutBans(vector<string> myChans)
{
   time_t now;
   time(&now);
   bool deleted=false;
   vector<string> deletedBans;
   deletedBans.clear();
   TiXmlNode * parent;
   TiXmlHandle docHandle (this->doc);
   TiXmlElement * elem,* elem2;
   elem = docHandle.FirstChild("trustyrc_moderation").FirstChild("bans").Element();
   if (elem) {
      for( elem=elem->FirstChildElement(); elem!=0; elem=elem->NextSiblingElement() ) {
         if ( Tools::isInVector(myChans,elem->ValueStr()) )
         {
            for (elem2=elem->FirstChildElement();elem2!=0;elem2=elem2->NextSiblingElement()) {
               if ( ((string)elem2->Attribute("duration")!="0") && ((now - ((Tools::strToInt(elem2->Attribute("timestamp")))+(Tools::strToInt(elem2->Attribute("duration"))))) >= 0 ))  {
                  deletedBans.push_back(IRCProtocol::unban(elem2->Attribute("mask"),"#"+elem->ValueStr()));
                  elem->RemoveChild(elem2);
                  deleted=true;
               }
            }
            parent = elem->Parent();
            if ( elem->FirstChild() == NULL )  {
               parent->RemoveChild(elem) ;
               deleted=true;
            }
         }
	   }
   }
   if(deleted) {
      this->doc->SaveFile();
   }
   return deletedBans;
}

/**
 *.Check if a user has the given access on the given channel.
 * The mode is the access one, in the xml file, not the one on the channel
 * @param channel Channel where to check access
 * @param mask Mask to check
 * @param level Level to check
 * @param b Pointer to the bot kerne
 * @return True if the user has the given access, else false
 */
bool Moderation::checkAccess(string channel,string mask,unsigned int level,BotKernel*b)
{
   pPlugin * ppAdm = b->getPlugin("admin");
   Admin*adm = NULL;
   if ( ppAdm != NULL ) {
      adm = (Admin*)ppAdm->object ;
      if ( adm->getUserLevel(channel,mask) >= level ) {
         return true; 
      }
   } 
   return false; 
}

/**
 * Check if a user is opped, has level >= 2 or is super admin
 * using "admin" and "usersinfos" modules
 * @param channel Channel where to check access
 * @param mask Mask to check access on the given channel
 * @param nick Nick to check access on the given channel
 * @param b Pointer to the bot kernel
 * @return True if the user, has op privileges, else false
 * @pre Admin and UsersInfos module must be loaded
 */
bool Moderation::hasOpPrivileges(string channel,string mask,string nick,BotKernel*b)
{
   pPlugin * ppAdm = b->getPlugin("admin");
   pPlugin * ppUser = b->getPlugin("usersinfos");
   Admin*adm = NULL;
   UsersInfos*ui = NULL;
   if ( ppAdm != NULL ) {
      adm = (Admin*)ppAdm->object ;
      if ( adm->isSuperAdmin(mask) ) {
         return true; 
      }
      if ( adm->getUserLevel(channel,mask) >= 2 ) {
         return true; 
      }
   }  
   if ( ppUser != NULL ) {
      ui = (UsersInfos*)ppUser->object ;
      if (ui->hasMode(channel,nick,'o')) {
         return true;
      }
   }
   return false;
}

/**
 * Check if a user has the given mode on a given channel using UsersInfos module
 * @param mode Mode to test for the user
 * @param channel Channel to check
 * @param nick Nick to check
 * @param b Pointer to the bot kernel
 * @pre "UsersInfos must be loaded
 * @return True if the user has the mode, else false 
 */
bool Moderation::checkMode(string channel,string nick,char mode,BotKernel*b)
{
   pPlugin * ppUser = b->getPlugin("usersinfos");
   UsersInfos*ui = NULL;
   if ( ppUser != NULL ) {
      ui = (UsersInfos*)ppUser->object ;
      if (ui->hasMode(channel,nick,mode)) {
         return true;
      }
      else {
         return false;
      } 
   }
   return false;
}

/**
 * Get a channel users list using "usersinfos" plugin
 * Users a stored in a string tab like that :
 * tab[0]=nick;
 * tab[1]=host;
 * tab[2]=ident;
 * tab[3]=status;
 * @param channel Channel to get the list
 * @param b Pointer to the bot kernel
 * @pre UsersInfos module must be loaded
 * @return A vector containing the users list
 */
vector<string*> Moderation::getChanUsersList(string channel,BotKernel*b)
{
   map<string,Channel*>* users;
   vector<string*> list;
   map<string,Channel*>::iterator fter ;
   pPlugin * ppUser = b->getPlugin("usersinfos");
   UsersInfos*ui = NULL;
   if ( ppUser != NULL ) {
      ui = (UsersInfos*)ppUser->object ;
      users = ui->getUsers();
      fter = users->find(channel);
      if ( fter != users->end() ) {
         return ((Channel*)fter->second)->getUsers();
      }
   }
   return list;
}

/**
 * Get rejoin attempts number for a channel
 * @param channel Channel for which we want to get rejoin attempts number
 * @return Rejoin attempts number
 */
unsigned int Moderation::getRejoinAttempts(string channel)
{
	map<string,int>::iterator fter = this->rejoinAttempts.find(channel);
	if ( fter != this->rejoinAttempts.end() ) {
		return fter->second ;
	}
	else {
		return 0;
	}
}

/**
 * Bump rejoin attempts for a channel
 * @param channel Channel for which we want to bump attempts number
 */
void Moderation::bumpRejoinAttempts(string channel)
{
	map<string,int>::iterator fter = this->rejoinAttempts.find(channel);
	if ( fter != this->rejoinAttempts.end() ) {
		this->rejoinAttempts[channel] += 1 ;
	}
	else {
		this->rejoinAttempts[channel] = 1 ;
	}
}

/**
 * Clear attempts for a channel
 * @param channel Channel for which we want to clear rejoin attempts
 */
void Moderation::clearRejoinAttempts(string channel)
{
	map<string,int>::iterator fter = this->rejoinAttempts.find(channel);
	if ( fter != this->rejoinAttempts.end() ) {
		this->rejoinAttempts.erase(fter);
	}
}

extern "C"
{
	Plugin *contruct_moderation(BotKernel*b)
	{
		return new Moderation(b);
	}
	void destroy_moderation(Plugin*p)
	{
		delete p;
	}
	bool unbanall (Message*m,Plugin*p,BotKernel*b)
   {
      Moderation*mod = (Moderation*)p;
      vector<string> bans;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            bans = mod->clearList(m->getSource()) ;
            b->send(IRCProtocol::applyModes(m->getSource(),bans,'-','b',4) ) ;
         }
      }
      return true;
   }
	bool bandel (Message*m,Plugin*p,BotKernel*b)
   {
      string mask = "";
      Moderation*mod = (Moderation*)p;
      if (m->isPublic() && (m->nbParts() == 5)) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            mask = mod->delBan(m->getSource(),Tools::strToInt(m->getPart(4)));
            if (mask!="") {
               b->send( IRCProtocol::unban(mask,m->getSource()) ) ;
            }
         }
      }
      return true;
   }
	bool baninfos (Message*m,Plugin*p,BotKernel*b)
   {
      Moderation*mod = (Moderation*)p;
      if (m->isPublic() && (m->nbParts() == 5)) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            b->send( IRCProtocol::sendNotices(m->getNickSender(),mod->banInfos(m->getSource(),Tools::strToInt(m->getPart(4)))) ) ;
         }
      }
      return true;
   }
	bool banlist (Message*m,Plugin*p,BotKernel*b)
   {
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            b->send( IRCProtocol::sendNotices(m->getNickSender(),Tools::gatherVectorElements(mod->getBanList(m->getSource())," ",3)) ) ;
         }
      }
      return true;
   }
	bool ban (Message*m,Plugin*p,BotKernel*b)
   {
      string mask,host;
      ConfigurationFile*cff = b->getCONFF();
      map<string,Channel*>* users;
      map<string,Channel*>::iterator fter ;
      vector<string> nicks;
      pPlugin * ppUser = b->getPlugin("usersinfos");
      UsersInfos*ui = NULL;
      Moderation*mod = (Moderation*)p;
      if ( ppUser != NULL )
      {
         ui = (UsersInfos*)ppUser->object;
         if (m->isPublic()) {
            if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
               if ( (m->nbParts() >= 7) && (m->getPart(5).length() <= 9) ) {
                  users = ui->getUsers();
                  fter = users->find(m->getSource());
                  if ( fter != users->end() ) {
                     host = ((Channel*)fter->second)->getHostByNick(m->getPart(4)) ;
                     if (host != "" ) {
                        mask ="*!*@"+ host ;
                        mod->addBan(m->getSource(),mask,Tools::strtimeToSeconds(m->getPart(5)),m->getSender(),Tools::vectorToString(m->getSplit()," ",6));
                        b->send(IRCProtocol::ban(mask,m->getSource()));
                        if ( cff->getValue(p->getName()+".autokick") == "1" ) {
                           b->send(IRCProtocol::kick(m->getPart(4),m->getSource(),"["+m->getPart(5)+"]"+Tools::vectorToString(m->getSplit()," ",6)) ) ;
                        }
                     }
                  }
               }
            }
         }
      }
      return true;
   }   
	bool banmask (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
      vector<string> nicks;
      vector<string*> users;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if ( (m->nbParts() >= 7) && (m->getPart(5).length() <= 9) ) {
               mod->addBan(m->getSource(),m->getPart(4),Tools::strtimeToSeconds(m->getPart(5)),m->getSender(),Tools::vectorToString(m->getSplit()," ",6));
               b->send(IRCProtocol::ban(m->getPart(4),m->getSource()));
               if ( cff->getValue(p->getName()+".autokick") == "1" ) {
                  users = mod->getChanUsersList(m->getSource(),b);
                  for(unsigned int i = 0 ; i < users.size() ; i ++ ) {
                     if ( Tools::ircMaskMatch(users[i][0]+"!"+users[i][2]+"@"+users[i][1],m->getPart(4)) && (users[i][0]!=b->getNick()) ) {
                        b->send(IRCProtocol::kick(users[i][0],m->getSource(),"["+m->getPart(5)+"]"+Tools::vectorToString(m->getSplit()," ",6)) ) ;
                     }
                  }
               }
            }
         }
      }
      return true;
   }
	bool op (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string> nicks;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if ( m->getSplit().size() >= 5 ) {
               for (unsigned int i = 4 ; i < m->getSplit().size(); i ++)
					   nicks.push_back(m->getPart(i));
            }
            else {
				   nicks.push_back(m->getNickSender());
			   }
            b->send(IRCProtocol::op(nicks,m->getSource()));
         }
      }
      return true;
   }
	bool unop (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string> nicks;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if ( m->getSplit().size() >= 5 ) {
               for (unsigned int i = 4 ; i < m->getSplit().size(); i ++)
                  if ( m->getPart(i) != b->getNick() ) {
		   			   nicks.push_back(m->getPart(i));
                  }
            }
            else {
		   		nicks.push_back(m->getNickSender());
		   	}
            b->send(IRCProtocol::unop(nicks,m->getSource()));
         }
      }
      return true;
   }
	bool voice (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string> nicks;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if ( m->getSplit().size() >= 5 ) {
               for (unsigned int i = 4 ; i < m->getSplit().size(); i ++)
	   				nicks.push_back(m->getPart(i));
            }
            else {
	   			nicks.push_back(m->getNickSender());
	   		}
            b->send(IRCProtocol::voice(nicks,m->getSource()));
         }
      }
      return true;
   }
	bool unvoice (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string> nicks;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if ( m->getSplit().size() >= 5 ) {
               for (unsigned int i = 4 ; i < m->getSplit().size(); i ++)
	   				nicks.push_back(m->getPart(i));
            }
            else {
	   			nicks.push_back(m->getNickSender());
	   		}
            b->send(IRCProtocol::unvoice(nicks,m->getSource()));
         }
      }
      return true;
   }
	bool topic (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string> nicks;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            b->send(IRCProtocol::changeTopic(m->getSource(),Tools::vectorToString(m->getSplit()," ",4)));
         }
      }
      return true;
   }
	bool kick (Message*m,Plugin*p,BotKernel*b)
   {
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( (m->getSplit().size() > 4) && (m->getPart(4)!=b->getNick() )) {
            if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
               b->send(IRCProtocol::kick(m->getPart(4),m->getSource(),Tools::vectorToString(m->getSplit()," ",5)));
            }
         }
      }
      return true;
   }
	bool masskick (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string> kicks;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            for (unsigned int i = 4 ; i < m->getSplit().size(); i ++) {
               if ( m->getPart(i) != b->getNick() )
                  kicks.push_back(IRCProtocol::kick(m->getPart(i),m->getSource(),"o/"));
            }
            b->getSysLog()->log("MASSKICK on "+m->getSource()+" (by "+m->getSender()+")",INFO);
            b->send(kicks);
         }
      }
      return true;
   }
	bool opall (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string*> users;
      Moderation*mod = (Moderation*)p;
      vector<string> opers;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            users = mod->getChanUsersList(m->getSource(),b);
            for (unsigned int i = 0 ; i < users.size() ; i ++ ) {
               if (!mod->checkMode(m->getSource(),users[i][0],'o',b)) {
                  opers.push_back(users[i][0]);
               }
            }
            b->getSysLog()->log("OPALL on "+m->getSource()+" (by "+m->getSender()+")",INFO);
            b->send(IRCProtocol::op(opers,m->getSource()));
         }
      }
      return true;
   }
	bool unopall (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string*> users;
      Moderation*mod = (Moderation*)p;
      vector<string> unopers;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            users = mod->getChanUsersList(m->getSource(),b);
            for (unsigned int i = 0 ; i < users.size() ; i ++ )
            {
               if ( (users[i][0] != b->getNick()) && (mod->checkMode(m->getSource(),users[i][0],'o',b)) ) {
                  unopers.push_back(users[i][0]);
               }
            }
            b->getSysLog()->log("UNOPALL on "+m->getSource()+" (by "+m->getSender()+")",INFO);
            b->send(IRCProtocol::unop(unopers,m->getSource()));
         }
      }
      return true;
   }
	bool voiceall (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string*> users;
      Moderation*mod = (Moderation*)p;
      vector<string> voices;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            users = mod->getChanUsersList(m->getSource(),b);
            for (unsigned int i = 0 ; i < users.size() ; i ++ ) {
               if (!mod->checkMode(m->getSource(),users[i][0],'v',b)) {
                  voices.push_back(users[i][0]);
               }
            }
            b->getSysLog()->log("VOICEALL on "+m->getSource()+" (by "+m->getSender()+")",INFO);
            b->send(IRCProtocol::voice(voices,m->getSource()));
         }
      }
      return true;
   }
	bool unvoiceall (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string*> users;
      Moderation*mod = (Moderation*)p;
      vector<string> unvoices;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            users = mod->getChanUsersList(m->getSource(),b);
            for (unsigned int i = 0 ; i < users.size() ; i ++ ) {
               if (mod->checkMode(m->getSource(),users[i][0],'v',b)) {
                  unvoices.push_back(users[i][0]);
               }
            }
            b->getSysLog()->log("UNVOICEALL on "+m->getSource()+" (by "+m->getSender()+")",INFO);
            b->send(IRCProtocol::unvoice(unvoices,m->getSource()));
         }
      }
      return true;
   }
	bool kickall (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string*> users;
      vector<string> kickeds;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            users = mod->getChanUsersList(m->getSource(),b);
            for (unsigned int i = 0 ; i < users.size() ; i ++ ) {
               if (users[i][0] != b->getNick()) {
                  kickeds.push_back(IRCProtocol::kick(users[i][0],m->getSource(),"o/"));
               }
            }
            b->getSysLog()->log("KICKALL on "+m->getSource()+" (by "+m->getSender()+")",INFO);
            b->send(kickeds);
         }
      }
      return true;
   }
	bool randomKick (Message*m,Plugin*p,BotKernel*b)
   {
      vector<string*> users;
      string nick;
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            users = mod->getChanUsersList(m->getSource(),b);
            b->getSysLog()->log("RANDOMKICK on "+m->getSource()+" (by "+m->getSender()+")",INFO);
            if (users.size() == 0 ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),"* Unable to do it now *"));
               return true;
            }
            nick = users[Tools::random(0,users.size()-1)][0] ;
            if ( nick == b->getNick() ) {
               b->send(IRCProtocol::sendMsg(m->getSource(),"* It was on me !!!! *"));
               return true; 
            }
            b->send(IRCProtocol::kick(nick,m->getSource(),b->getCONFF()->getValue(p->getName()+".rdmkickmsg")));
         }
      }
      return true;
   }
	bool modeHandler (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
      map<string,Channel*>* users;
      map<string,Channel*>::iterator fter ;
      string*infos;
      string mask="";
      pPlugin * ppUser = b->getPlugin("usersinfos");
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      UsersInfos*ui = NULL;
      if ((cff->getValue(p->getName()+".revenge")=="1")&&(ppAdm != NULL)&&(ppUser != NULL)&&(m->getNickSender()!=b->getNick()) ) {
         adm = (Admin*)ppAdm->object ;
         ui = (UsersInfos*)ppUser->object;
		   if ( (m->getSplit().size() > 4) && (adm->getUserLevel(m->getSource(),m->getSender())<2) && (!adm->isSuperAdmin(m->getSender())))
		   {
		      string modes = m->getPart(3) ;
			   char sign = '\0';
			   unsigned int nicksIndex = 4 ;
			   for (unsigned int i = 0 ; i < modes.length() ; i ++ ) {
			   	if ((modes[i]=='+')||(modes[i]=='-')) {
			   		sign = modes[i];
			   	}
			   	else {
                  users = ui->getUsers();
                  fter = users->find(m->getSource());
                  if ( fter != users->end() ) {
                     if ((sign=='-')&&((modes[i]=='v')||(modes[i]=='o'))) { 
                        infos = ((Channel*)fter->second)->getInfosByNick(m->getPart(nicksIndex));
                        if ( infos != NULL)
                        {
                           mask = infos[0]+"!"+infos[2]+"@"+infos[1] ;
                           if ( (modes[i]=='v') && (adm->isSuperAdmin(mask)||(adm->getUserLevel(m->getSource(),mask) >= 1)) ) {
                              b->send("MODE "+m->getSource()+" -o+v "+m->getNickSender()+" "+m->getPart(nicksIndex));
                           }
                           else if ((modes[i]=='o') && (adm->isSuperAdmin(mask)||(adm->getUserLevel(m->getSource(),mask) >= 2) )) {   
                              b->send("MODE "+m->getSource()+" -o+bo "+m->getNickSender()+" "+"*!*@"+m->getHostSender()+" "+m->getPart(nicksIndex));
                              b->send( IRCProtocol::kick(m->getNickSender(),m->getSource(),"Do not unop this user !")  );
                           }
                        }
                     }
                     else if (sign=='+'&&modes[i]=='b') {
                        mask = m->getPart(nicksIndex) ;
                        if ( adm->maskIsSuperAdmin(mask) || (adm->getMaskLevel(m->getSource(),mask) >= 2) ) {
                           b->send("MODE "+m->getSource()+" -ob+b "+m->getNickSender()+" "+mask+" "+"*!*@"+m->getHostSender());
                           b->send( IRCProtocol::kick(m->getNickSender(),m->getSource(),"Do not ban this user !")  );
                        }
                     }
                  }
			   		nicksIndex++;
			   	}
			   }
		   }
      }  
      return true;
   }
	bool modeHandlerProtect (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
		Moderation*mod = (Moderation*)p;
      pPlugin * ppAdm = b->getPlugin("admin");
      pPlugin * ppUser = b->getPlugin("usersinfos");
      Admin*adm = NULL;
      UsersInfos*ui = NULL;
      string modes ;
      char sign;
		if(Tools::isInVector(Tools::stringToVector(cff->getValue(p->getName()+".protectmodes"),",",0),m->getSource())) {
   		adm = (Admin*)ppAdm->object ;
         ui = (UsersInfos*)ppUser->object;
			if ( (!adm->isSuperAdmin(m->getSender()))&&(!mod->checkAccess(m->getSource(),m->getSender(),2,b))&&(m->getNickSender()!=b->getNick()) ) {
            modes = m->getPart(3) ;
			   sign = '\0';
			   for (unsigned int i = 0 ; i < modes.length() ; i ++ ) {
			   	if ((modes[i]=='+')||(modes[i]=='-')) {
			   		sign = modes[i];
			   	}
               else {
                  if ( ((string)ui->getPrefixes()+"b").find(modes[i]) == string::npos ) {
                     if (sign=='+') {
                        b->send("MODE "+m->getSource()+" -"+modes[i]);
                     }
                     else if (sign=='-') {
                        b->send("MODE "+m->getSource()+" +"+modes[i]);
                     }
                  }   
               }
            }
			}
		}
      return true;
   }
	bool joinHandler (Message*m,Plugin*p,BotKernel*b)
   {
      Moderation * mod = (Moderation*)p;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      vector<string> nick;
      if (m->getNickSender()==b->getNick()) {
         mod->clearRejoinAttempts(m->getSource());
         return true;
      }
      if ( mod->isBanned(m->getSource(),m->getSender())) { // banned ?
         b->send(IRCProtocol::ban("*!*@"+m->getHostSender(),m->getSource())) ;
         b->send(IRCProtocol::kick(m->getNickSender(),m->getSource(),"You are banned from this channel"));
         return true;
      }
      if (Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".autoop"),",",0),m->getSource())) { // chan autotop ?
	   	nick.push_back(m->getNickSender());
         b->send(IRCProtocol::op(nick,m->getSource()));
	   }
      if (Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".autovoice"),",",0),m->getSource())) { // chan autovoice ?
		   nick.push_back(m->getNickSender());
     	   b->send(IRCProtocol::voice(nick,m->getSource()));
	   }
      if ( ppAdm != NULL ) {
         adm = (Admin*)ppAdm->object ;
         if ( adm->getUserLevel(m->getSource(),m->getSender()) == 1 ) { // voice acces ?
            b->send(IRCProtocol::voice(m->getNickSender(),m->getSource()));
         }
         else if ( adm->getUserLevel(m->getSource(),m->getSender()) >= 2 ) { // op acces ?
            b->send(IRCProtocol::op(m->getNickSender(),m->getSource()));
         }
      }
      return true;
   }
	bool partHandler (Message*m,Plugin*p,BotKernel*b)
   {
      Moderation * mod = (Moderation*)p;
      map<string,Channel*>* users;
      map<string,Channel*>::iterator fter ;
      pPlugin * ppUser = b->getPlugin("usersinfos");
      UsersInfos*ui = NULL;
      if ( ppUser != NULL ) {
         ui = (UsersInfos*)ppUser->object ;
         users = ui->getUsers();
         fter = users->find(m->getSource());
         if ( fter != users->end() )
         {
            if ( (((Channel*)fter->second)->getUsers().size() == 1) && (!mod->checkMode(m->getSource(),b->getNick(),'o',b)))
            {
               b->send(IRCProtocol::leaveChannel(m->getSource(),"..."));
               b->send(IRCProtocol::joinChannel(m->getSource()));
            }
         }
      }
      return true;
   }
	bool quitHandler (Message*m,Plugin*p,BotKernel*b)
   {
      Moderation * mod = (Moderation*)p;
      map<string,Channel*>* users;
      pPlugin * ppUser = b->getPlugin("usersinfos");
      UsersInfos*ui = NULL;
      if ( ppUser != NULL ) {
         ui = (UsersInfos*)ppUser->object ;
         users = ui->getUsers();
         for (map<string,Channel*>::const_iterator iter = users->begin();iter!=users->end();++iter)
		   { 
            if ( (((Channel*)iter->second)->getUsers().size() == 1) && (!mod->checkMode(iter->first,b->getNick(),'o',b)))
            {
               b->send(IRCProtocol::leaveChannel(iter->first,"..."));
               b->send(IRCProtocol::joinChannel(iter->first));
            }
		   }
      }
      return true;
   }
	bool kickHandler (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
      Moderation * mod = (Moderation*)p;
      map<string,Channel*>* users;
      map<string,Channel*>::iterator fter ;
      string mask ="";
      pPlugin * ppUser = b->getPlugin("usersinfos");
      UsersInfos*ui = NULL;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      if( m->getPart(3) == b->getNick() )
      {
         if ( cff->getValue(p->getName()+".rejoin_when_kicked") == "1" ) {
            b->send(IRCProtocol::joinChannel(m->getSource()));
         }
         b->getSysLog()->log("Kicked from "+m->getSource()+" (by "+m->getSender()+")",INFO);
      }
      else
      {
         if ( ppUser != NULL ) {
            ui = (UsersInfos*)ppUser->object ;
            users = ui->getUsers();
            fter = users->find(m->getSource());
            if ( fter != users->end() )
            {
               if ( (((Channel*)fter->second)->getUsers().size() == 1) && (!mod->checkMode(m->getSource(),b->getNick(),'o',b)))
               {
                  b->send(IRCProtocol::leaveChannel(m->getSource(),"..."));
                  b->send(IRCProtocol::joinChannel(m->getSource()));
               }
               else if ((cff->getValue(p->getName()+".revenge")=="1")&&(ppAdm != NULL)&&( m->getNickSender() != b->getNick() )) {
                  adm = (Admin*)ppAdm->object ;
                  string*infos = ((Channel*)fter->second)->getInfosByNick(m->getPart(3));
                  if ((infos != NULL) && (adm->getUserLevel(m->getSource(),m->getSender())<2) && (!adm->isSuperAdmin(m->getSender())))
                  {
                     mask = infos[0]+"!"+infos[2]+"@"+infos[1] ;
                     if ((adm->isSuperAdmin(mask)||(adm->getUserLevel(m->getSource(),mask) >= 2))) {
                        b->send("MODE "+m->getSource()+" -o+b "+m->getNickSender()+" "+"*!*@"+m->getHostSender());
                        b->send( IRCProtocol::kick(m->getNickSender(),m->getSource(),"Do not kick this user !")  );
                     }
                  }
               }
            }
         }
      }
      return true;
   }
	bool autoop (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if (!Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".autoop"),",",0),m->getSource())) {
               cff->setValue(p->getName()+".autoop",cff->getValue(p->getName()+".autoop")+","+m->getSource());
               b->send(IRCProtocol::sendMsg(m->getSource(),"done."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"already autoop"));
            }
         }
      }
      return true;
   }
	bool unautoop (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
      Moderation*mod = (Moderation*)p;
      vector<string> chans;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if (Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".autoop"),",",0),m->getSource())) {
               chans = Tools::stringToVector(cff->getValue(p->getName()+".autoop"),",",0) ;
               Tools::delStrFromVector(&chans,m->getSource()) ;
               cff->setValue(p->getName()+".autoop",Tools::vectorToString(chans,",",0));
               b->send(IRCProtocol::sendMsg(m->getSource(),"done."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"not autoop"));
            }
         }
      }
      return true;
   }
	bool autovoice (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if (!Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".autovoice"),",",0),m->getSource())) {
               cff->setValue(p->getName()+".autovoice",cff->getValue(p->getName()+".autovoice")+","+m->getSource());
               b->send(IRCProtocol::sendMsg(m->getSource(),"done."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"already autovoice"));
            }
         }
      }
      return true;
   }
	bool unautovoice (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cff = b->getCONFF();
      Moderation*mod = (Moderation*)p;
      vector<string> chans;
      if (m->isPublic()) {
         if ( mod->hasOpPrivileges(m->getSource(),m->getSender(),m->getNickSender(),b)) {
            if (Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".autovoice"),",",0),m->getSource())) {
               chans = Tools::stringToVector(cff->getValue(p->getName()+".autovoice"),",",0) ;
               Tools::delStrFromVector(&chans,m->getSource()) ;
               cff->setValue(p->getName()+".autovoice",Tools::vectorToString(chans,",",0));
               b->send(IRCProtocol::sendMsg(m->getSource(),"done."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"not autovoice"));
            }
         }
      }
      return true;
   }
	bool protecttopic (Message*m,Plugin*p,BotKernel*b)
   {
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      ConfigurationFile*cff = b->getCONFF();
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
			adm = (Admin*)ppAdm->object ;
         if ((adm->isSuperAdmin(m->getSender()))||(mod->checkAccess(m->getSource(),m->getSender(),2,b))) {
            if (!Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".protecttopic"),",",0),m->getSource())) {
               cff->setValue(p->getName()+".protecttopic",cff->getValue(p->getName()+".protecttopic")+","+m->getSource());
               b->send(IRCProtocol::sendMsg(m->getSource(),"done."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"already protected"));
            }
         }
      }
      return true;
   }
	bool unprotecttopic (Message*m,Plugin*p,BotKernel*b)
   {
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      ConfigurationFile*cff = b->getCONFF();
      Moderation*mod = (Moderation*)p;
      vector<string> chans;
      if (m->isPublic()) {
			adm = (Admin*)ppAdm->object ;
         if ((adm->isSuperAdmin(m->getSender()))||(mod->checkAccess(m->getSource(),m->getSender(),2,b))) {
            if (Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".protecttopic"),",",0),m->getSource())) {
               chans = Tools::stringToVector(cff->getValue(p->getName()+".protecttopic"),",",0) ;
               Tools::delStrFromVector(&chans,m->getSource()) ;
               cff->setValue(p->getName()+".protecttopic",Tools::vectorToString(chans,",",0));
               b->send(IRCProtocol::sendMsg(m->getSource(),"done."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"not protected"));
            }
         }
      }
      return true;
   }
//////////////////////////////////////////////////////
	bool protectmodes (Message*m,Plugin*p,BotKernel*b)
   {
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      ConfigurationFile*cff = b->getCONFF();
      Moderation*mod = (Moderation*)p;
      if (m->isPublic()) {
			adm = (Admin*)ppAdm->object ;
         if ((adm->isSuperAdmin(m->getSender()))||(mod->checkAccess(m->getSource(),m->getSender(),2,b))) {
            if (!Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".protectmodes"),",",0),m->getSource())) {
               cff->setValue(p->getName()+".protectmodes",cff->getValue(p->getName()+".protectmodes")+","+m->getSource());
               b->send(IRCProtocol::sendMsg(m->getSource(),"done."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"already protected"));
            }
         }
      }
      return true;
   }
	bool unprotectmodes (Message*m,Plugin*p,BotKernel*b)
   {
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      ConfigurationFile*cff = b->getCONFF();
      Moderation*mod = (Moderation*)p;
      vector<string> chans;
      if (m->isPublic()) {
			adm = (Admin*)ppAdm->object ;
         if ((adm->isSuperAdmin(m->getSender()))||(mod->checkAccess(m->getSource(),m->getSender(),2,b))) {
            if (Tools::isInVector(Tools::stringToVector(b->getCONFF()->getValue(p->getName()+".protectmodes"),",",0),m->getSource())) {
               chans = Tools::stringToVector(cff->getValue(p->getName()+".protectmodes"),",",0) ;
               Tools::delStrFromVector(&chans,m->getSource()) ;
               cff->setValue(p->getName()+".protectmodes",Tools::vectorToString(chans,",",0));
               b->send(IRCProtocol::sendMsg(m->getSource(),"done."));
            }
            else {
               b->send(IRCProtocol::sendMsg(m->getSource(),"not protected"));
            }
         }
      }
      return true;
   }
//////////////////////////////////////////////////////
	bool clearOutBans (Message*m,Plugin*p,BotKernel*b)
   {
      pPlugin * ppUser = b->getPlugin("usersinfos");
      UsersInfos*ui = NULL;  
      Moderation*mod = (Moderation*)p;
      vector<string> bans,myChans;
      map<string,Channel*>::iterator it ;
      if ( ppUser != NULL ) {
         ui = (UsersInfos*)ppUser->object ;
         map<string,Channel*>* users = ui->getUsers();
         it = users->begin();
         while ( it != users->end() )
         {
            if (mod->checkMode((string)it->first,b->getNick(),'o',b)) {
               myChans.push_back(((string)it->first).substr(1)) ;
            }
            it++;
         }
         bans = mod->clearOutBans(myChans) ;
         for (unsigned int i = 0 ; i < bans.size() ; i ++ ) {
            b->send(bans[i]);
         } 
      }
      return true;
   }
	bool invite (Message*m,Plugin*p,BotKernel*b)
   {
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      if ( (ppAdm != NULL) && m->isPrivate() && (m->nbParts() == 6) ) {
         adm = (Admin*)ppAdm->object ;
         if ( (adm->getUserLevel(m->getPart(5),m->getSender())>=2) || (adm->isSuperAdmin(m->getSender())) ) {
            b->send(IRCProtocol::invite(m->getPart(4),m->getPart(5))) ;
         }
      }
      return true;
   }
	bool rejoinChan (Message*m,Plugin*p,BotKernel*b)
   {
      Moderation*mod = (Moderation*)p;
		ConfigurationFile * cf = b->getCONFF();
      string nbAttempts = cf->getValue(p->getName()+".rejoin_ban_attempts");
      if ( (nbAttempts == "") || (nbAttempts == "0") || (mod->getRejoinAttempts(m->getMessage())<Tools::strToUnsignedInt(nbAttempts)) ) {
         mod->bumpRejoinAttempts(m->getMessage());
         b->send(IRCProtocol::joinChannel(m->getMessage()));
      }
      else {
         b->getSysLog()->log("I'm banned from "+m->getMessage()+" and will no longer try to rejoin (did "+nbAttempts+" attempts)",WARNING); 
      }
      return true;
   }
	bool bannedHandler (Message*m,Plugin*p,BotKernel*b)
   {
      ConfigurationFile*cf = b->getCONFF();
      if ( cf->getValue(p->getName()+".rejoin_when_banned") == "1" ) {
         Message msg(m->getPart(3));
         if(b->addCountDown(p,rejoinChan,&msg,Tools::strToUnsignedInt(cf->getValue(p->getName()+".rejoin_ban_time")),5)==NULL) {
            b->getSysLog()->log("Couldn't launch chan rejoin after ban (max countdowns reached)",INFO);   
         }
      }
      return true;
   }
	bool topicJoin (Message*m,Plugin*p,BotKernel*b)
	{
      pPlugin * ppUser = b->getPlugin("usersinfos");
      UsersInfos*ui = NULL;  
      map<string,Channel*>::iterator it ;
      if ( ppUser != NULL ) {
         ui = (UsersInfos*)ppUser->object ;
         map<string,Channel*>* users = ui->getUsers();
      	it = users->find(m->getPart(3));
      	if ( it != users->end() ) {
         	((Channel*)it->second)->setTopic(Tools::vectorToString(m->getSplit()," ",4).substr(1));
      	}
      }
      return true;
	}
	bool topicHandler(Message*m,Plugin*p,BotKernel*b)
	{
      ConfigurationFile*cff = b->getCONFF();
		Moderation*mod = (Moderation*)p;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      pPlugin * ppUser = b->getPlugin("usersinfos");
      UsersInfos*ui = NULL;  
      map<string,Channel*>::iterator it ;
		if(Tools::isInVector(Tools::stringToVector(cff->getValue(p->getName()+".protecttopic"),",",0),m->getSource())) {
      	if ( (ppUser != NULL)&&(ppAdm != NULL) ) {
				adm = (Admin*)ppAdm->object ;
         	ui = (UsersInfos*)ppUser->object ;
				if ( (!adm->isSuperAdmin(m->getSender()))&&(!mod->checkAccess(m->getSource(),m->getSender(),2,b))&&(m->getNickSender()!=b->getNick()) ) {
         		map<string,Channel*>* users = ui->getUsers();
      			it = users->find(m->getSource());
      			if ( it != users->end() ) {
						b->send(IRCProtocol::changeTopic(m->getSource(),((Channel*)it->second)->getTopic()));
      			}
				}
				else {
         		map<string,Channel*>* users = ui->getUsers();
      			it = users->find(m->getSource());
      			if ( it != users->end() ) {
         			((Channel*)it->second)->setTopic(Tools::vectorToString(m->getSplit()," ",3).substr(1));
      			}
				}
      	}
		}
      return true;
	}
}
