/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file moderation.h
 * @brief Moderation header file
 */

#ifndef MODULE_H
#define MODULE_H
#include "../plugin.h"
#include "../botkernel.h"

#ifndef HAVE_TINYXML
#include "../tinyxml/tinyxml.h"
#else 
#include <tinyxml.h>
#endif


#include "admin.h"
#include "usersinfos.h"
#include <time.h>
#include <map>
using namespace std;

/** @brief Channel moderation
 *
 * This plugin allow channels moderation (op, voice, kick,bans etc ...) 
 * with acces (if admin plugin is loaded)
 */

class Moderation : public Plugin
{
   private:
      /// Represent the xml document
      TiXmlDocument * doc ;
      /// Represent documents's root
      TiXmlNode* root;
      /// Initialize the XML file
      void initFile();
      /// Stores rejoin attempts
		map<string,int> rejoinAttempts;
   public:
      /// Constructor
	   Moderation(BotKernel*);
      /// add a ban on a channel
      bool addBan(string,string,unsigned int,string,string);
      /// del a ban on a channel
      string delBan(string,unsigned int);
      /// tell if a user is banned
      bool isBanned(string,string);
      /// give ban list on a channel
      vector<string> getBanList(string);
      /// give details about a ban
      vector<string> banInfos(string,unsigned int);
      /// clear all bans for a channel and return them (to apply it on the chan)
      vector<string> clearList(string);
      /// CLear all outdated bans for all channels and return them (to apply it on chans)
      vector<string> clearOutBans(vector<string>);
      /// Check access on a chan for a user
      bool checkAccess(string,string,unsigned int,BotKernel*);
      /// Check if a user is opped, has level >= 2 or is super admin
      bool hasOpPrivileges(string,string,string,BotKernel*);
      /// Get a channel users list
      vector<string*> getChanUsersList(string,BotKernel*);
      /// Check if a user has the given mode on a given channel using UsersInfos module
      bool checkMode(string,string,char,BotKernel*);
      /// Get rejoin attempts number for a channel
      unsigned int getRejoinAttempts(string);
      /// Bump rejoin attempts for a channel
      void bumpRejoinAttempts(string);
      /// Clear attempts for a channel
      void clearRejoinAttempts(string);
};


#endif

