/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file module.cpp
 * @brief Module implementation file
 */

#include "module.h"

/**
 * Constructor
 */ 
Module::Module(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "bot modules management";
	this->version = VERSION;
	this->name = "module";
	this->bindFunction("load",IN_COMMAND_HANDLER,"load",0,10);
	this->bindFunction("unload",IN_COMMAND_HANDLER,"unload",0,10);
	this->bindFunction("loadnocheck",IN_COMMAND_HANDLER,"loadnocheck",0,10);
	this->bindFunction("unloadnocheck",IN_COMMAND_HANDLER,"unloadnocheck",0,10);
	this->bindFunction("listmodules",IN_COMMAND_HANDLER,"listmodules",0,10);
	this->bindFunction("listlibs",IN_COMMAND_HANDLER,"listlibs",0,10);
	this->bindFunction("moduleinfos",IN_COMMAND_HANDLER,"moduleinfos",0,10);
   this->addRequirement("admin");
}

extern "C"
{
	Plugin *contruct_module(BotKernel*b)
	{
		return new Module(b);
	}
	void destroy_module(Plugin*p)
	{
		delete p;
	}
   bool load (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm ;
      pPlugin * pp = b->getPlugin("admin");
      if ( pp != NULL ) {
         adm = (Admin*) pp->object;
         if ( (adm!=NULL) && m->isPrivate() && ( m->nbParts() == 5 ) && adm->isSuperAdmin( m->getSender()) ) {
            if(b->loadPlugin(m->getPart(4),true)) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" loaded"));
               b->getSysLog()->log(m->getPart(4)+" loaded by "+m->getSender(),INFO);
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" loading failed."));
               b->getSysLog()->log(m->getPart(4)+" loading failed (by "+m->getSender()+")",WARNING);
            }
         }
      }
		return true;
   }
   bool unload (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm ;
      pPlugin * pp = b->getPlugin("admin");   
      if ( pp != NULL ) {
         adm = (Admin*) pp->object;
         if ( (adm!=NULL) && m->isPrivate() && ( m->nbParts() == 5 ) && adm->isSuperAdmin( m->getSender()) ) {
            if(b->unloadPlugin(m->getPart(4),true)) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" unloaded"));
               b->getSysLog()->log(m->getPart(4)+" unloaded by "+m->getSender(),INFO);
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" unloading failed."));
               b->getSysLog()->log(m->getPart(4)+" unloading failed (by "+m->getSender()+")",WARNING);
            }
         }
      }   
		return true;
   }
   bool loadnocheck (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm ;
      pPlugin * pp = b->getPlugin("admin");
      if ( pp != NULL ) {
         adm = (Admin*) pp->object;
         if ( (adm!=NULL) && m->isPrivate() && ( m->nbParts() == 5 ) && adm->isSuperAdmin( m->getSender()) ) {
            if(b->loadPlugin(m->getPart(4),false)) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" loaded"));
               b->getSysLog()->log(m->getPart(4)+" loaded by "+m->getSender(),INFO);
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" loading failed."));
               b->getSysLog()->log(m->getPart(4)+" loading failed (by "+m->getSender()+")",WARNING);
            }
         }
      }
		return true;
   }
   bool unloadnocheck (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm ;
      pPlugin * pp = b->getPlugin("admin");   
      if ( pp != NULL ) {
         adm = (Admin*) pp->object;
         if ( (adm!=NULL) && m->isPrivate() && ( m->nbParts() == 5 ) && adm->isSuperAdmin( m->getSender()) ) {
            if(b->unloadPlugin(m->getPart(4),false)) {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" unloaded"));
               b->getSysLog()->log(m->getPart(4)+" unloaded by "+m->getSender(),INFO);
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4)+" unloading failed."));
               b->getSysLog()->log(m->getPart(4)+" unloading failed (by "+m->getSender()+")",WARNING);
            }
         }
      }   
		return true;
   }
   bool listmodules (Message*m,Plugin*p,BotKernel*b)
   {
      Admin * adm ;
      pPlugin * pp = b->getPlugin("admin");
      if ( pp != NULL )
      {
         adm = (Admin*) pp->object;
         if( (adm!=NULL) && m->isPrivate() && adm->isSuperAdmin( m->getSender()) ) {
            b->send(IRCProtocol::sendNotices(m->getNickSender(),Tools::gatherVectorElements(b->getPluginsList()," ",4)));
         }
      }   
		return true;
   }
   bool listlibs (Message*m,Plugin*p,BotKernel*b) {
      DIR *dp;
      struct dirent *ep;
      const unsigned int NBPATHS = 3;
      string paths[NBPATHS] = {b->getDatasDir()+"../plugins",PLUGINSDIR,"plugins"};
      vector<string> plugins;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      if ((ppAdm!=NULL) && m->isPrivate() ) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            for(unsigned int i = 0 ; i < NBPATHS ; i ++ ) {
               plugins.clear();  
               dp = opendir (paths[i].c_str());
               if (dp != NULL) {
                  while ((ep = readdir(dp))) {
                     if ( ((string)ep->d_name != ".") && ((string)ep->d_name !="..")) {
                        plugins.push_back(ep->d_name);
                     }
                  }
                  closedir (dp);
               }
               else {
                  plugins.push_back("* Couldn't open the directory ("+paths[i]+") *");
               }
               b->send(IRCProtocol::sendNotices(m->getNickSender(),Tools::gatherVectorElements(plugins," ",4)));
            }
         }
      }
      return true;
   }
   bool moduleinfos (Message*m,Plugin*p,BotKernel*b) {
      pPlugin * pp = NULL;
      pPlugin * ppAdm = b->getPlugin("admin");
      Admin*adm = NULL;
      Plugin * plug;
      if ((ppAdm!=NULL) && m->isPrivate() && (m->nbParts() == 5)) {
         adm = (Admin*)ppAdm->object ;
         if (adm->isSuperAdmin(m->getSender()) ) {
            pp = b->getPlugin(m->getPart(4) ) ;
            if ( pp != NULL ) {
               plug = pp->object;
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4) +" ("+plug->getVersion()+") by "+plug->getAuthor()+" : "+plug->getDescription()));
            }
            else {
               b->send(IRCProtocol::sendNotice(m->getNickSender(),m->getPart(4) +" not loaded"));
            }
         }
      }
      return true;
   }
}
