/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file ping.cpp
 * @brief Ping implementation file
 */

#include "ping.h"

/**
 * Constructor
 */
Ping::Ping(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Make the bot answer \"pong\" to \"ping\" message";
	this->version = VERSION;
	this->name = "ping";
   time_t now;
   time(&now);
	this->bindFunction("PING",IN_FIRST_WORD,"pinged",0,10);
	this->bindFunction("120",IN_LOOP,"checkConnection",(now),10);
	this->bindFunction("PONG",IN_TYPE_HANDLER,"pongMe",0,10);
   this->setPonged(true);
}

/**
 * Set ponged value
 * @param value Ponged value
 */
void Ping::setPonged(bool value)
{
   this->ponged = value;
}

/**
 * Get ponged value
 * @return Ponged value
 */
bool Ping::getPonged()
{
   return this->ponged ;
}

extern "C"
{
	Plugin *contruct_ping(BotKernel*b)
	{
		return new Ping(b);
	}
	void destroy_ping(Plugin*p)
	{
		delete p;
	}
	bool pinged (Message*m,Plugin*p,BotKernel*b)
	{
   	b->send(IRCProtocol::pong(m->getPart(1).substr(1))) ;
		return true;
	}
	bool checkConnection(Message*m,Plugin*p,BotKernel*b)
	{
      Ping* ping = (Ping*)p;
      if (ping->getPonged() == false) {
         b->getSysLog()->log("Disconnected (ping timed out)",WARNING);
         ping->setPonged(true);
         b->setConnected(false);
      }  
      else
      {
         ping->setPonged(false);
         b->send( IRCProtocol::ping(b->getNick()) );
      }
		return true;
	}
	bool pongMe(Message*m,Plugin*p,BotKernel*b)
	{
      Ping* ping = (Ping*)p;
      ping->setPonged(true);
		return true;
	}
}
