/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file postconnect.cpp
 * @brief PostConnect implementation file
 */

#include "postconnect.h"

/**
 * Constructor
 */
PostConnect::PostConnect(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Auto actions performed after server connection";
	this->version = VERSION;
	this->name = "postconnect";
	this->bindFunction("376",IN_TYPE_HANDLER,"onEndOfMOTD",0,40);
	this->bindFunction("422",IN_TYPE_HANDLER,"onEndOfMOTD",0,40); // No Motd ! (motd file missing)
	this->bindFunction("433",IN_TYPE_HANDLER,"secondaryNick",0,10);
	this->bindFunction("NICK",IN_TYPE_HANDLER,"nick_changed",0,10);
   this->resetNickRetreiveAttempts();
}

/**
 * get nick take attemtps
 * @return attempts number
 */
unsigned int PostConnect::getNickRetreiveAttempts()
{
   return this->nickRetreiveAttempts;
}

/**
 * Increase nick take attempts
 */
void PostConnect::bumpNickRetreiveAttempts() 
{
   this->nickRetreiveAttempts += 1 ;
}

/**
 * Rest nick take attempts
 */ 
void PostConnect::resetNickRetreiveAttempts()
{
   this->nickRetreiveAttempts = 0 ;
}

extern "C"
{
	Plugin *contruct_postconnect(BotKernel*b)
	{
		return new PostConnect(b);
	}
	void destroy_postconnect(Plugin*p)
	{
		delete p;
	}
	bool nick_changed(Message*m,Plugin*p,BotKernel*b)
	{
      PostConnect * pc = (PostConnect*) p;
      if ( m->getPart(2).substr(1) == b->getNick() ) {
         pc->resetNickRetreiveAttempts() ;
      }  
      return true;
   }
	bool getMyFirstNick(Message*m,Plugin*p,BotKernel*b)
	{
      PostConnect * pc = (PostConnect*) p;
      pc->bumpNickRetreiveAttempts() ;
		ConfigurationFile * cf = b->getCONFF();
      string nbAttempts = cf->getValue(p->getName()+".retreive_nick_attempts");
      if ( (nbAttempts == "") || (nbAttempts == "0") || (pc->getNickRetreiveAttempts()<=Tools::strToUnsignedInt(nbAttempts)) ) {
         b->setNick(cf->getValue("kernel.nick"));
         b->send(IRCProtocol::changeNick(cf->getValue("kernel.nick")));
      }
      else {
         b->setNick(cf->getValue("kernel.nick2"));
      }
		return true;
	}
	bool secondaryNick(Message*m,Plugin*p,BotKernel*b)
	{
		ConfigurationFile * cf = b->getCONFF();
		b->getSysLog()->log("Nick "+cf->getValue("kernel.nick")+" already in use",INFO);
      b->setNick(cf->getValue("kernel.nick2"));
      b->send(IRCProtocol::changeNick(cf->getValue("kernel.nick2")));
      if ( cf->getValue(p->getName()+".retreive_nick") == "1" ) {
         if(b->addCountDown(p,getMyFirstNick,m,Tools::strToUnsignedInt(cf->getValue(p->getName()+".retreive_nick_time")),5)==NULL) {
            b->getSysLog()->log("Couldn't launch nick take back (max countdowns reached)",INFO); 
         }            
      }
		return true;
	}
	bool onEndOfMOTD(Message*m,Plugin*p,BotKernel*b)
	{
		vector<string> temp ;
		ConfigurationFile * cf = b->getCONFF();
		if ( cf->getValue(p->getName()+".qauth") != "")
		{
			temp = Tools::stringToVector(cf->getValue(p->getName()+".qauth"),",",0);	
			if (temp.size() == 2 ) {
				b->getSysLog()->log("QAUTH",INFO);
				b->send("PRIVMSG Q@CServe.quakenet.org :AUTH "+ temp[0] +" "+ temp[1]);
			}
			else
				b->getSysLog()->log("Wrong QAUTH value : "+cf->getValue(p->getName()+".qauth"),WARNING);
		}
		else if ( cf->getValue(p->getName()+".qauthx") != "")
		{
			temp = Tools::stringToVector(cf->getValue(p->getName()+".qauthx"),",",0);	
			if (temp.size() == 2 ) {
				b->getSysLog()->log("QAUTH, MODE +x",INFO);
				b->send("PRIVMSG Q@CServe.quakenet.org :AUTH "+ temp[0] +" "+ temp[1]);
				b->send("MODE "+ b->getNick()+ " +x");
			}
			else
				b->getSysLog()->log("Wrong QAUTHX value : "+cf->getValue(p->getName()+".qauthx"),WARNING);
		}
		if ( cf->getValue(p->getName()+".uauth") != "")
		{
			temp = Tools::stringToVector(cf->getValue(p->getName()+".uauth"),",",0);	
			if (temp.size() == 2 ) {
				b->getSysLog()->log("UAUTH",INFO);
				b->send("PRIVMSG X@channels.undernet.org :LOGIN "+ temp[0] +" "+ temp[1]);
			}
			else
				b->getSysLog()->log("Wrong UAUTH value : "+cf->getValue(p->getName()+".uauth"),WARNING);
		}
		else if ( cf->getValue(p->getName()+".uauthx") != "")
		{
			temp = Tools::stringToVector(cf->getValue(p->getName()+".uauthx"),",",0);	
			if (temp.size() == 2 ) {
				b->getSysLog()->log("UAUTH, MODE +x",INFO);
				b->send("PRIVMSG X@channels.undernet.org :LOGIN "+ temp[0] +" "+ temp[1]);
				b->send("MODE "+ b->getNick() + " +x");
			}
			else
				b->getSysLog()->log("Wrong UAUTHX value : "+cf->getValue(p->getName()+".uauthx"),WARNING);
		}
		if ( cf->getValue(p->getName()+".nsauth") != "")
		{
				b->getSysLog()->log("NSAUTH",INFO);
				b->send("PRIVMSG NickServ :IDENTIFY "+ cf->getValue(p->getName()+".nsauth"));
		}
		temp = Tools::stringToVector(cf->getValue(p->getName()+".raw"),",",0);
		for ( unsigned int i = 0 ; i < temp.size() ; i ++ ) {
			b->getSysLog()->log("Executing raw action : " + temp[i],INFO);
			b->send(temp[i]);
		}
    	sleep(Tools::strToInt(cf->getValue(p->getName()+".sleep")));
		temp = Tools::gatherVectorElements(Tools::stringToVector(cf->getValue(p->getName()+".join"),",",0),",",4);
		for ( unsigned int i = 0 ; i < temp.size() ; i ++ ) {
			b->getSysLog()->log("Going to join " + temp[i],INFO);
			b->send(IRCProtocol::joinChannel(temp[i]));
		}
		return true;
	}
}
