/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file quotes.h
 * @brief Quotes header file
 */

#ifndef CLASS_H
#define CLASS_H
#include "../plugin.h"
#include "../botkernel.h"

#ifndef HAVE_TINYXML
#include "../tinyxml/tinyxml.h"
#else 
#include <tinyxml.h>
#endif


#include "admin.h"
#include <iostream>
using namespace std;

/** @brief Quotes management (storage and access)
 *
 * This plugin stores quotes in a XML file and give access to them
 */

class Quotes : public Plugin
{
private:
   /// Represent the xml document
   TiXmlDocument * doc ;
   /// Represent documents's root
   TiXmlNode* root;
   /// Quotes number
   unsigned int nbQuotes;
   /// Return nodes's child's number
   unsigned int getNbChilds(TiXmlNode*);
public:
   /// Constructor
	Quotes(BotKernel*);
   /// Add a quote
   void addQuote(string,string);
   /// Return a quote
   string getQuote(unsigned int);
   /// Return a random quote
   string getRandomQuote();
   /// Search quotes according to a pattern
   vector<string> searchQuote(string);
   /// Delete a quote
   bool delQuote(unsigned int);
   /// Return last inserted quote
   string getLastQuote();
   /// Return informations aboute a quote
   string quoteInfos(unsigned int);
};


#endif

