/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file remotecontrol.h
 * @brief RemoteControl header file
 */

#ifndef REMOTECONTROL_H
#define REMOTECONTROL_H

#include "../cppthread.h"
#include "../plugin.h"
#include "../botkernel.h"

#include <iostream>
using namespace std;

/** @brief Plugin that allow remote TCP control
 *
 * This plugins make the bot listen on a port.
 * A client can connect to the bot with a telnet command
 * and can then control the bot by sending commands
 */

class RemoteControl : public Plugin
{
private:
   /// Thread object   
   CPPThread* pt;
   /// Socket used to accept connections
   int sockfd;
   /// Array to store connections
   int * clients; 
   /// Port used for connection
   unsigned int MYPORT;
   /// Maximum clients that will be able to connect
   unsigned int MAXCLIENTS  ;
   /// Listen backlog
   static const int BACKLOG = 5;
   /// Max data size fo communication
   static const int MAXDATASIZE = 100 ;
public:
   /// Constructor
	RemoteControl(BotKernel*);
   /// Destructor
   ~RemoteControl();
   /// Prepare FD for select and return highest sock
   void setSocketList(struct timeval*,fd_set*);
   /// manage incomming connections
   int manageNewConnection(int);
   /// TCP server to receive connections
   void* tcpServer(BotKernel*);
};


#endif

