/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file survey.h
 * @brief Survey header file
 */

#ifndef SURVEY_H
#define CLASS_H
#include "../plugin.h"
#include "../botkernel.h"
#include "admin.h"
#include <iostream>
using namespace std;

/// Plugin object and header storage
typedef struct 
{
   string channel;
   string question;
   unsigned int time;
   vector<string> answers;
   vector<int> results;
   vector<string> voters;   
   vector<plugin_function> functions;
   plugin_function countDown;
} struct_survey;

/** @brief This plugin manages surveys
 *
 * This plugin manages surveys
 */

class Survey : public Plugin
{
private:
   /// Surveys storage
   vector<struct_survey> surveys;
   /// Get an answer's id
   int getAnswerId(vector<string>,string);
   /// Test if a survey is running
   bool surveyRunning(string); 
public:
   /// Constructor
	Survey(BotKernel*);
   /// Launch a survey
   bool launchSurvey(string,string,unsigned int,vector<string>);
   /// Stop a survey
   bool stopSurvey(string);
   /// Vote
   bool vote(string,string,string);
   /// Finish a survey
   vector<string> finishSurvey(string);
   /// Get survey's functions
   vector<plugin_function> getSurveyFunctions(string);
   /// Set survey's functions
   bool setSurveyFunctions(string,vector<plugin_function>);
   /// get countdown pointer
   plugin_function getCountDown(string);
   /// set countdown pointer
   bool setCountDown(string,plugin_function);
};


#endif

