/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file trad.cpp
 * @brief Trad implementation file
 */

#include "trad.h"

/**
 * Constructor
 */ 
Trad::Trad(BotKernel*b)
{
	this->author = "eponyme";
	this->description = "Provides a command to translate a sentence from a language to an other using translate.google.com";
	this->version = VERSION;
	this->name = "trad";
	this->bindFunction("trad",IN_COMMAND_HANDLER,"trad",0,10);
}

extern "C"
{
	Plugin *contruct_trad(BotKernel*b)
	{
		return new Trad(b);
	}
	void destroy_trad(Plugin*p)
	{
		delete p;
	}
	bool trad (Message*m,Plugin*p,BotKernel*b)
	{
      string buffer,text,from,to ="";
      const string flag = "<div id=result_box dir=\"ltr\">";
      string::size_type pos;
      Socket sock = Socket();
      if ((m->isPublic()) && (m->nbParts() >= 7) )
      {
         from = Tools::urlencode(m->getPart(4));
         to = Tools::urlencode(m->getPart(5));
         text = Tools::urlencode(Tools::vectorToString(m->getSplit()," ",6) );
         if (!sock.connectSock(80,"translate.google.com","")) {
            b->send(IRCProtocol::sendMsg(m->getSource(),"* Unable to connect to translate.google.com *")) ;
            return true;
         }
         sock.sendStr("GET /translate_t?text="+text+"&langpair="+from+"%7C"+to+"&hl=fr HTTP/1.1\nHost: translate.google.com\n\n");
         while (buffer.find("</html>")==string::npos) {
            buffer += sock.receive() ;
         }
         pos = buffer.find(flag);
         if ( pos == string::npos) {
	         b->send(IRCProtocol::sendMsg(m->getSource(),"* Error while parsing result *")) ;
            return true;
         }
         buffer = buffer.substr(pos,buffer.length()-pos);
         pos = buffer.find("</div>");
         if ( pos == string::npos) {
	         b->send(IRCProtocol::sendMsg(m->getSource(),"* Error while parsing result *")) ;
            return true;
         }
         buffer = buffer.substr(flag.length(),pos-flag.length());
         b->send(IRCProtocol::sendMsg(m->getSource(),"Translation : "+Tools::cleanHTML(buffer))) ;
      }
		return true;
	}
}
