/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file socket.h
 * @brief Socket header file
 */

#ifndef SOCKETIRC_H
#define SOCKETIRC_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <string.h>
#include <iostream>
#include <string>
#include "tools.h"
using namespace std ;

/** @brief Class that manage the connection with the server
 *
 * This class manage the connection with the irc server.
 * It uses a socket for communication
 */

class Socket 
{
   public :
		/// Constructor
      Socket();
		/// Destructor
      ~Socket();
		/// Send a string to the server
      bool sendStr(string);
		/// Close the socket
      bool closeSock();
		/// Connect the socket to the server
      bool connectSock(int,string,string);
      /// Get connection state 
      bool getState();
		/// Receive a string from the server
      string receive();
   private :
      /// Socket state
      bool state;
		/// Socket handle
      int mySock ;
};

#endif
