/*
#########################################################################
#
#  This file is part of trustyRC.
#
#  trustyRC, fully modular IRC robot 
#  Copyright (C) 2006-2008 Nicoleau Fabien 
#
#  trustyRC is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  trustyRC is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with trustyRC.  If not, see <http://www.gnu.org/licenses/>.
#
#########################################################################
*/

/** @file tools.h
 * @brief Tools header file
 */

#ifndef TOOLS_H
#define TOOLS_H
#include <time.h>
#include <sstream>
#include <fstream>
#include <vector>
#include <string>
#include <stdlib.h>
#include <fnmatch.h>
using namespace std;

/** @brief Class that provides tools for programmation
 *
 * Provide different static methods for regular treatment.
 * Those methods can be used everywhere in the code an
 * doesn't need an object instantiation.
 */

class Tools
{
   public :
	/// Constructor
   Tools();
	/// Destructor
   ~Tools();
   /// Convert an ascii string to an hexadecimal string
   static string asciiToHexa(string);
   /// Convert an hexadecimal string to an ascii string
   static string hexaToAscii(string);
	/// Convert a int to a string
   static string intToStr(int);
	/// Convert a double to a string
   static string doubleToStr(double);
	/// Convert a string to a double
   static double strToDouble(string);
	/// Convert a string to a int
   static int strToInt(string);
	/// Convert a string to an unsigned int
   static unsigned int strToUnsignedInt(string);
   /// Convert a string time to seconds
   static unsigned int strtimeToSeconds(string);
	/// Convert a string to lower case
   static string to_lower(string);
	/// Convert a string to upper case
   static string to_upper(string);
	/// Give a random number
   static int random(int min,int max);
	/// Convert a vector to a string
   static string vectorToString(vector<string>,string,unsigned int start = 0);
	/// Convert a string to a vector
	static vector<string> stringToVector(string,string,unsigned int start = 0);
   /// Gather vector elements 
   static vector<string> gatherVectorElements(vector<string>,string,unsigned int);
   /// Escape a char in a string
   static string escapeChar(string,char);
   /// Log a string in a file
   static void log(string,string,bool timestamp=true,bool truncate=false);
   /// Encode a string to a URL format
   static string urlencode (string);
   /// Clear accents from a sentence
   static string clearAccents(string);
   /// Clean HTML code in a string 
   static string cleanHTML(string);
   /// Tell if a string is in a vector
   static bool isInVector(vector<string>,string);  
   /// Withdraw a string from a vector
   static void delStrFromVector(vector<string>*,string);
   /// Parse Quake colors from a string 
	static string parseQ3Colors(string);
	/// Tell if an irc host match to a mask
	static bool ircMaskMatch(string,string);
   /// Tell if two masks match
   static int masksMatch(char*,char*);
   /// Copy a file
   static bool copyFile(string,string);
};

#endif
